@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2020-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2020-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -44,17 +44,17 @@ void run_sort_keys_benchmark(benchmark::State& state,
 {
     using offset_type = int;
     using key_type    = Key;
-    typedef hipError_t (*sort_func)(void*,
-                                    size_t&,
-                                    const key_type*,
-                                    key_type*,
-                                    int,
-                                    int,
-                                    offset_type*,
-                                    offset_type*,
-                                    int,
-                                    int,
-                                    hipStream_t);
+    using sort_func   = hipError_t (*)(void*,
+                                     size_t&,
+                                     const key_type*,
+                                     key_type*,
+                                     int,
+                                     int,
+                                     offset_type*,
+                                     offset_type*,
+                                     int,
+                                     int,
+                                     hipStream_t);
 
     sort_func func_ascending = &hipcub::DeviceSegmentedRadixSort::SortKeys<key_type, offset_type*>;
     sort_func func_descending
@@ -83,18 +83,10 @@ void run_sort_keys_benchmark(benchmark::State& state,
     }
     offsets.push_back(size);
 
-    std::vector<key_type> keys_input;
-    if(std::is_floating_point<key_type>::value)
-    {
-        keys_input
-            = benchmark_utils::get_random_data<key_type>(size, (key_type)-1000, (key_type) + 1000);
-    } else
-    {
-        keys_input
-            = benchmark_utils::get_random_data<key_type>(size,
-                                                         std::numeric_limits<key_type>::min(),
-                                                         std::numeric_limits<key_type>::max());
-    }
+    std::vector<key_type> keys_input = benchmark_utils::get_random_data<key_type>(
+        size,
+        benchmark_utils::generate_limits<key_type>::min(),
+        benchmark_utils::generate_limits<key_type>::max());
 
     offset_type* d_offsets;
     HIP_CHECK(hipMalloc(&d_offsets, (segments_count + 1) * sizeof(offset_type)));
@@ -188,19 +180,19 @@ void run_sort_pairs_benchmark(benchmark::State& state,
     using offset_type = int;
     using key_type    = Key;
     using value_type  = Value;
-    typedef hipError_t (*sort_func)(void*,
-                                    size_t&,
-                                    const key_type*,
-                                    key_type*,
-                                    const value_type*,
-                                    value_type*,
-                                    int,
-                                    int,
-                                    offset_type*,
-                                    offset_type*,
-                                    int,
-                                    int,
-                                    hipStream_t);
+    using sort_func   = hipError_t (*)(void*,
+                                     size_t&,
+                                     const key_type*,
+                                     key_type*,
+                                     const value_type*,
+                                     value_type*,
+                                     int,
+                                     int,
+                                     offset_type*,
+                                     offset_type*,
+                                     int,
+                                     int,
+                                     hipStream_t);
 
     sort_func func_ascending
         = &hipcub::DeviceSegmentedRadixSort::SortPairs<key_type, value_type, offset_type*>;
@@ -230,18 +222,10 @@ void run_sort_pairs_benchmark(benchmark::State& state,
     }
     offsets.push_back(size);
 
-    std::vector<key_type> keys_input;
-    if(std::is_floating_point<key_type>::value)
-    {
-        keys_input
-            = benchmark_utils::get_random_data<key_type>(size, (key_type)-1000, (key_type) + 1000);
-    } else
-    {
-        keys_input
-            = benchmark_utils::get_random_data<key_type>(size,
-                                                         std::numeric_limits<key_type>::min(),
-                                                         std::numeric_limits<key_type>::max());
-    }
+    std::vector<key_type> keys_input = benchmark_utils::get_random_data<key_type>(
+        size,
+        benchmark_utils::generate_limits<key_type>::min(),
+        benchmark_utils::generate_limits<key_type>::max());
 
     std::vector<value_type> values_input(size);
     std::iota(values_input.begin(), values_input.end(), 0);
