@@ -1,7 +1,7 @@
 /******************************************************************************
  * Copyright (c) 2010-2011, Duane Merrill.  All rights reserved.
  * Copyright (c) 2011-2018, NVIDIA CORPORATION.  All rights reserved.
- * Modifications Copyright (c) 2017-2024, Advanced Micro Devices, Inc.  All rights reserved.
+ * Modifications Copyright (c) 2017-2025, Advanced Micro Devices, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -33,7 +33,7 @@
 #include "../../../config.hpp"
 #include "../../../util_deprecated.hpp"
 
-#include <cub/device/device_reduce.cuh>
+#include <cub/device/device_reduce.cuh> // IWYU pragma: export
 
 BEGIN_HIPCUB_NAMESPACE
 
@@ -241,6 +241,34 @@ public:
         return ArgMax(d_temp_storage, temp_storage_bytes, d_in, d_out, num_items, stream);
     }
 
+    template<typename InputIteratorT,
+             typename OutputIteratorT,
+             typename ReductionOpT,
+             typename TransformOpT,
+             typename T,
+             typename NumItemsT>
+    HIPCUB_RUNTIME_FUNCTION
+    static hipError_t TransformReduce(void*           d_temp_storage,
+                                      size_t&         temp_storage_bytes,
+                                      InputIteratorT  d_in,
+                                      OutputIteratorT d_out,
+                                      NumItemsT       num_items,
+                                      ReductionOpT    reduction_op,
+                                      TransformOpT    transform_op,
+                                      T               init,
+                                      hipStream_t     stream = 0)
+    {
+        return hipCUDAErrorTohipError(::cub::DeviceReduce::TransformReduce(d_temp_storage,
+                                                                           temp_storage_bytes,
+                                                                           d_in,
+                                                                           d_out,
+                                                                           num_items,
+                                                                           reduction_op,
+                                                                           transform_op,
+                                                                           init,
+                                                                           stream));
+    }
+
     template<typename KeysInputIteratorT,
              typename UniqueOutputIteratorT,
              typename ValuesInputIteratorT,
