@@ -1,7 +1,7 @@
 /******************************************************************************
  * Copyright (c) 2010-2011, Duane Merrill.  All rights reserved.
  * Copyright (c) 2011-2018, NVIDIA CORPORATION.  All rights reserved.
- * Modifications Copyright (c) 2017-2024, Advanced Micro Devices, Inc.  All rights reserved.
+ * Modifications Copyright (c) 2017-2025, Advanced Micro Devices, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -36,8 +36,9 @@
 #include "../iterator/arg_index_input_iterator.hpp"
 #include "../thread/thread_operators.hpp"
 
-#include <rocprim/device/device_reduce.hpp>
-#include <rocprim/device/device_reduce_by_key.hpp>
+#include <rocprim/device/device_reduce.hpp> // IWYU pragma: export
+#include <rocprim/device/device_reduce_by_key.hpp> // IWYU pragma: export
+#include <rocprim/iterator/transform_iterator.hpp> // IWYU pragma: export
 
 #include <hip/hip_bfloat16.h> // hip_bfloat16
 #include <hip/hip_fp16.h> // __half
@@ -185,8 +186,7 @@ public:
             d_out,
             init,
             num_items,
-            ::hipcub::detail::convert_binary_result_type<T, InputIteratorT, OutputIteratorT>(
-                reduction_op),
+            ::hipcub::detail::convert_binary_result_type<InputIteratorT, T>(reduction_op),
             stream,
             HIPCUB_DETAIL_DEBUG_SYNC_VALUE);
     }
@@ -419,6 +419,37 @@ public:
                       debug_synchronous);
     }
 
+    template<typename InputIteratorT,
+             typename OutputIteratorT,
+             typename ReductionOpT,
+             typename TransformOpT,
+             typename T,
+             typename NumItemsT>
+    HIPCUB_RUNTIME_FUNCTION
+    static hipError_t TransformReduce(void*           d_temp_storage,
+                                      size_t&         temp_storage_bytes,
+                                      InputIteratorT  d_in,
+                                      OutputIteratorT d_out,
+                                      NumItemsT       num_items,
+                                      ReductionOpT    reduction_op,
+                                      TransformOpT    transform_op,
+                                      T               init,
+                                      hipStream_t     stream = 0)
+    {
+        using TransformInputIteratorT = ::rocprim::transform_iterator<InputIteratorT, TransformOpT>;
+
+        return ::rocprim::reduce(
+            d_temp_storage,
+            temp_storage_bytes,
+            TransformInputIteratorT(d_in, transform_op),
+            d_out,
+            init,
+            num_items,
+            ::hipcub::detail::convert_binary_result_type<TransformInputIteratorT, T>(reduction_op),
+            stream,
+            HIPCUB_DETAIL_DEBUG_SYNC_VALUE);
+    }
+
     template<typename KeysInputIteratorT,
              typename UniqueOutputIteratorT,
              typename ValuesInputIteratorT,
