@@ -1,7 +1,7 @@
 /******************************************************************************
  * Copyright (c) 2010-2011, Duane Merrill.  All rights reserved.
  * Copyright (c) 2011-2018, NVIDIA CORPORATION.  All rights reserved.
- * Modifications Copyright (c) 2017-2024, Advanced Micro Devices, Inc.  All rights reserved.
+ * Modifications Copyright (c) 2017-2025, Advanced Micro Devices, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -37,10 +37,11 @@
 #include "../thread/thread_operators.hpp"
 #include "../util_sync.hpp"
 #include "device_reduce.hpp"
-#include "rocprim/type_traits.hpp"
 
-#include <rocprim/device/device_segmented_reduce.hpp>
+#include <rocprim/device/device_segmented_reduce.hpp> // IWYU pragma: export
+#include <rocprim/type_traits.hpp> // IWYU pragma: export
 
+#include <chrono>
 #include <iterator>
 #include <limits>
 
@@ -55,15 +56,15 @@ template<class Config,
          class OffsetIterator,
          class ResultType,
          class BinaryFunction>
-__global__ __launch_bounds__(
-    ::rocprim::detail::device_params<Config>()
-        .reduce_config.block_size) void segmented_arg_minmax_kernel(InputIterator  input,
-                                                                    OutputIterator output,
-                                                                    OffsetIterator begin_offsets,
-                                                                    OffsetIterator end_offsets,
-                                                                    BinaryFunction reduce_op,
-                                                                    ResultType     initial_value,
-                                                                    ResultType     empty_value)
+ROCPRIM_KERNEL __launch_bounds__(::rocprim::detail::device_params<Config>()
+                                     .reduce_config.block_size)
+    void segmented_arg_minmax_kernel(InputIterator  input,
+                                     OutputIterator output,
+                                     OffsetIterator begin_offsets,
+                                     OffsetIterator end_offsets,
+                                     BinaryFunction reduce_op,
+                                     ResultType     initial_value,
+                                     ResultType     empty_value)
 {
     // each block processes one segment
     ::rocprim::detail::segmented_reduce<Config>(input,
@@ -118,8 +119,7 @@ inline hipError_t segmented_arg_minmax(void*          temporary_storage,
                                        hipStream_t    stream)
 {
     using input_type = typename std::iterator_traits<InputIterator>::value_type;
-    using result_type =
-        typename ::rocprim::invoke_result_binary_op<input_type, BinaryFunction>::type;
+    using result_type = ::rocprim::accumulator_t<BinaryFunction, input_type>;
 
     using config = ::rocprim::detail::wrapped_reduce_config<Config, result_type>;
 
