@@ -1,7 +1,7 @@
 /******************************************************************************
  * Copyright (c) 2011, Duane Merrill.  All rights reserved.
  * Copyright (c) 2011-2018, NVIDIA CORPORATION.  All rights reserved.
- * Modifications Copyright (c) 2021-2024, Advanced Micro Devices, Inc.  All rights reserved.
+ * Modifications Copyright (c) 2021-2025, Advanced Micro Devices, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -50,8 +50,7 @@ BEGIN_HIPCUB_NAMESPACE
 class GridBarrier
 {
 protected :
-
-    typedef unsigned int SyncFlag;
+    using SyncFlag = unsigned int;
 
     // Counters in global device memory
     SyncFlag* d_sync;
@@ -61,13 +60,14 @@ public:
     /**
      * Constructor
      */
-    GridBarrier() : d_sync(NULL) {}
+    GridBarrier() : d_sync(nullptr) {}
 
-     /**
+    /**
      * @typedef SyncFlag
      * @brief Synchronize
      */
-    __device__ __forceinline__ void Sync() const
+    __device__ __forceinline__
+    void Sync() const
     {
         volatile SyncFlag *d_vol_sync = d_sync;
 
@@ -153,7 +153,7 @@ public:
         if (d_sync)
         {
             retval = hipFree(d_sync);
-            d_sync = NULL;
+            d_sync = nullptr;
         }
         sync_bytes = 0;
         return retval;
