@@ -1,7 +1,7 @@
 /******************************************************************************
  * Copyright (c) 2010-2011, Duane Merrill.  All rights reserved.
  * Copyright (c) 2011-2018, NVIDIA CORPORATION.  All rights reserved.
- * Modifications Copyright (c) 2017-2024, Advanced Micro Devices, Inc.  All rights reserved.
+ * Modifications Copyright (c) 2017-2025, Advanced Micro Devices, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -51,8 +51,11 @@
 
 // Device
 #include "device/device_adjacent_difference.hpp"
+#include "device/device_copy.hpp"
+#include "device/device_for.hpp"
 #include "device/device_histogram.hpp"
 #include "device/device_memcpy.hpp"
+#include "device/device_merge.hpp"
 #include "device/device_merge_sort.hpp"
 #include "device/device_partition.hpp"
 #include "device/device_radix_sort.hpp"
@@ -102,7 +105,10 @@
 // Util
 #include "tuple.hpp"
 #include "util_allocator.hpp"
+#include "util_device.hpp"
+#include "util_macro.hpp"
 #include "util_ptx.hpp"
+#include "util_temporary_storage.hpp"
 #include "util_type.hpp"
 
 #endif // HIPCUB_ROCPRIM_HIPCUB_HPP_
