@@ -1,7 +1,7 @@
 /******************************************************************************
  * Copyright (c) 2010-2011, Duane Merrill.  All rights reserved.
  * Copyright (c) 2011-2018, NVIDIA CORPORATION.  All rights reserved.
- * Modifications Copyright (c) 2021-2024, Advanced Micro Devices, Inc.  All rights reserved.
+ * Modifications Copyright (c) 2021-2025, Advanced Micro Devices, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -32,12 +32,15 @@
 
 #include "../../config.hpp"
 
-#include <rocprim/detail/various.hpp>
-#include <rocprim/thread/radix_key_codec.hpp>
-#include <rocprim/types/future_value.hpp>
+#include <rocprim/detail/various.hpp> // IWYU pragma: export
+#include <rocprim/thread/radix_key_codec.hpp> // IWYU pragma: export
+#include <rocprim/type_traits.hpp> // IWYU pragma: export
+#include <rocprim/types/future_value.hpp> // IWYU pragma: export
 
-#include <hip/hip_fp16.h>
 #include <hip/hip_bfloat16.h>
+#include <hip/hip_fp16.h>
+#include <hip/hip_runtime.h>
+#include <hip/hip_vector_types.h>
 
 #include <limits>
 #include <type_traits>
@@ -51,19 +54,7 @@ using NullType = ::rocprim::empty_type;
 #endif
 
 #ifndef HIPCUB_IS_INT128_ENABLED
-    #if defined(__HIPCC_RTC__)
-        #if defined(__HIPCC_RTC_INT128__)
-            #define HIPCUB_IS_INT128_ENABLED 1
-        #endif // !defined(__HIPCC_RTC_INT128__)
-    #else // !defined(__HIPCC_RTC__)
-        #if HIP_VERSION >= 50400000
-            #if(HIPCUB_HOST_COMPILER == HIPCUB_HOST_COMPILER_GCC)                         \
-                || (HIPCUB_HOST_COMPILER == HIPCUB_HOST_COMPILER_CLANG) || defined(__ICC) \
-                || defined(__GNUC__) || defined(__clang__)
-                #define HIPCUB_IS_INT128_ENABLED 1
-            #endif // GCC || CLANG || ICC
-        #endif // VER >= 5.4
-    #endif // !defined(__HIPCC_RTC__)
+    #define HIPCUB_IS_INT128_ENABLED 1
 #endif // !defined(HIPCUB_IS_INT128_ENABLED)
 
 template<bool B, typename T, typename F> struct
@@ -240,16 +231,23 @@ struct AlignBytes
     };
 
     /// The "truly aligned" type
-    typedef T Type;
+    using Type = T;
 };
 
 // Specializations where host C++ compilers (e.g., 32-bit Windows) may disagree
 // with device C++ compilers (EDG) on types passed as template parameters through
 // kernel functions
 
-#define __HIPCUB_ALIGN_BYTES(t, b)         \
-    template <> struct AlignBytes<t>    \
-    { enum { ALIGN_BYTES = b }; typedef __align__(b) t Type; };
+    #define __HIPCUB_ALIGN_BYTES(t, b)                  \
+        template<>                                      \
+        struct AlignBytes<t>                            \
+        {                                               \
+            enum                                        \
+            {                                           \
+                ALIGN_BYTES = b                         \
+            };                                          \
+            using Type = __attribute__((aligned(b))) t; \
+        };
 
 __HIPCUB_ALIGN_BYTES(short4, 8)
 __HIPCUB_ALIGN_BYTES(ushort4, 8)
@@ -301,28 +299,26 @@ struct UnitWord
     };
 
     /// Biggest shuffle word that T is a whole multiple of and is not larger than the alignment of T
-    typedef typename std::conditional<IsMultiple<int>::IS_MULTIPLE,
-        unsigned int,
-        typename std::conditional<IsMultiple<short>::IS_MULTIPLE,
-            unsigned short,
-            unsigned char>::type>::type         ShuffleWord;
+    using ShuffleWord =
+        typename std::conditional<IsMultiple<int>::IS_MULTIPLE,
+                                  unsigned int,
+                                  typename std::conditional<IsMultiple<short>::IS_MULTIPLE,
+                                                            unsigned short,
+                                                            unsigned char>::type>::type;
 
     /// Biggest volatile word that T is a whole multiple of and is not larger than the alignment of T
-    typedef typename std::conditional<IsMultiple<long long>::IS_MULTIPLE,
-        unsigned long long,
-        ShuffleWord>::type                      VolatileWord;
+    using VolatileWord = typename std::
+        conditional<IsMultiple<long long>::IS_MULTIPLE, unsigned long long, ShuffleWord>::type;
 
     /// Biggest memory-access word that T is a whole multiple of and is not larger than the alignment of T
-    typedef typename std::conditional<IsMultiple<longlong2>::IS_MULTIPLE,
-        ulonglong2,
-        VolatileWord>::type                     DeviceWord;
+    using DeviceWord = typename std::
+        conditional<IsMultiple<longlong2>::IS_MULTIPLE, ulonglong2, VolatileWord>::type;
 
     /// Biggest texture reference word that T is a whole multiple of and is not larger than the alignment of T
-    typedef typename std::conditional<IsMultiple<int4>::IS_MULTIPLE,
+    using TextureWord = typename std::conditional<
+        IsMultiple<int4>::IS_MULTIPLE,
         uint4,
-        typename std::conditional<IsMultiple<int2>::IS_MULTIPLE,
-            uint2,
-            ShuffleWord>::type>::type           TextureWord;
+        typename std::conditional<IsMultiple<int2>::IS_MULTIPLE, uint2, ShuffleWord>::type>::type;
 };
 
 
@@ -330,20 +326,20 @@ struct UnitWord
 template <>
 struct UnitWord <float2>
 {
-    typedef int         ShuffleWord;
-    typedef unsigned long long   VolatileWord;
-    typedef unsigned long long   DeviceWord;
-    typedef float2      TextureWord;
+    using ShuffleWord  = int;
+    using VolatileWord = unsigned long long;
+    using DeviceWord   = unsigned long long;
+    using TextureWord  = float2;
 };
 
 // float4 specialization workaround (for SM10-SM13)
 template <>
 struct UnitWord <float4>
 {
-    typedef int         ShuffleWord;
-    typedef unsigned long long  VolatileWord;
-    typedef ulonglong2          DeviceWord;
-    typedef float4              TextureWord;
+    using ShuffleWord  = int;
+    using VolatileWord = unsigned long long;
+    using DeviceWord   = ulonglong2;
+    using TextureWord  = float4;
 };
 
 
@@ -351,10 +347,10 @@ struct UnitWord <float4>
 template <>
 struct UnitWord <char2>
 {
-    typedef unsigned short      ShuffleWord;
-    typedef unsigned short      VolatileWord;
-    typedef unsigned short      DeviceWord;
-    typedef unsigned short      TextureWord;
+    using ShuffleWord  = unsigned short;
+    using VolatileWord = unsigned short;
+    using DeviceWord   = unsigned short;
+    using TextureWord  = unsigned short;
 };
 
 
@@ -362,11 +358,98 @@ template <typename T> struct UnitWord<volatile T> : UnitWord<T> {};
 template <typename T> struct UnitWord<const T> : UnitWord<T> {};
 template <typename T> struct UnitWord<const volatile T> : UnitWord<T> {};
 
-
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 
+/******************************************************************************
+ * Vector type inference utilities.
+ ******************************************************************************/
 
+template<typename T, int vec_elements>
+struct GenericCubVector
+{
+    static_assert(!sizeof(T), "CubVector can only have 1-4 elements");
+};
 
+enum
+{
+    /// The maximum number of elements in HIP vector types
+    MAX_VEC_ELEMENTS = 4,
+};
+
+template<typename T>
+struct GenericCubVector<T, 1>
+{
+    T x;
+
+    using BaseType = T;
+    using Type     = GenericCubVector<T, 1>;
+};
+
+template<typename T>
+struct GenericCubVector<T, 2>
+{
+    T x;
+    T y;
+
+    using BaseType = T;
+    using Type     = GenericCubVector<T, 2>;
+};
+
+template<typename T>
+struct GenericCubVector<T, 3>
+{
+    T x;
+    T y;
+    T z;
+
+    using BaseType = T;
+    using Type     = GenericCubVector<T, 3>;
+};
+
+template<typename T>
+struct GenericCubVector<T, 4>
+{
+    T x;
+    T y;
+    T z;
+    T w;
+
+    using BaseType = T;
+    using Type     = GenericCubVector<T, 4>;
+};
+
+template<typename T, int vec_elements>
+struct CubVectorType
+{
+    // Fallback on GenericCubVector
+    using Type = GenericCubVector<T, vec_elements>;
+};
+
+template<typename T, int vec_elements>
+using CubVector = typename CubVectorType<T, vec_elements>::Type;
+
+#define HIPCUB_DEFINE_VECTOR_TYPE(base_type, vec_type)        \
+    template<int vec_elements>                                \
+    struct CubVectorType<base_type, vec_elements>             \
+    {                                                         \
+        using Type = HIP_vector_type<vec_type, vec_elements>; \
+    };
+
+HIPCUB_DEFINE_VECTOR_TYPE(char, char)
+HIPCUB_DEFINE_VECTOR_TYPE(unsigned char, unsigned char)
+HIPCUB_DEFINE_VECTOR_TYPE(short, short)
+HIPCUB_DEFINE_VECTOR_TYPE(ushort, ushort)
+HIPCUB_DEFINE_VECTOR_TYPE(int, int)
+HIPCUB_DEFINE_VECTOR_TYPE(uint, uint)
+HIPCUB_DEFINE_VECTOR_TYPE(long, long)
+HIPCUB_DEFINE_VECTOR_TYPE(unsigned long, unsigned long)
+HIPCUB_DEFINE_VECTOR_TYPE(long long, long long)
+HIPCUB_DEFINE_VECTOR_TYPE(unsigned long long, unsigned long long)
+HIPCUB_DEFINE_VECTOR_TYPE(float, float)
+HIPCUB_DEFINE_VECTOR_TYPE(double, double)
+HIPCUB_DEFINE_VECTOR_TYPE(bool, unsigned char)
+
+#undef HIPCUB_DEFINE_VECTOR_TYPE
 
 /******************************************************************************
  * Wrapper types
@@ -379,7 +462,7 @@ template <typename T>
 struct Uninitialized
 {
     /// Biggest memory-access word that T is a whole multiple of and is not larger than the alignment of T
-    typedef typename UnitWord<T>::DeviceWord DeviceWord;
+    using DeviceWord = typename UnitWord<T>::DeviceWord;
 
     static constexpr std::size_t DATA_SIZE = sizeof(T);
     static constexpr std::size_t WORD_SIZE = sizeof(DeviceWord);
@@ -395,19 +478,18 @@ struct Uninitialized
     }
 };
 
-
 /******************************************************************************
  * Simple type traits utilities.
  *
  * For example:
  *     Traits<int>::CATEGORY             // SIGNED_INTEGER
- *     Traits<NullType>::NULL_TYPE       // true
+ *     Traits<NullType>::nullptr_TYPE       // true
  *     Traits<uint4>::CATEGORY           // NOT_A_NUMBER
  *     Traits<uint4>::PRIMITIVE;         // false
  *
  ******************************************************************************/
 
- #ifndef DOXYGEN_SHOULD_SKIP_THIS    // Do not document
+#ifndef DOXYGEN_SHOULD_SKIP_THIS // Do not document
 
 /**
  * \brief Basic type traits categories
@@ -424,15 +506,19 @@ enum Category
 /**
  * \brief Basic type traits
  */
-template <Category _CATEGORY, bool _PRIMITIVE, bool _NULL_TYPE, typename _UnsignedBits, typename T>
+template<Category _CATEGORY,
+         bool     _PRIMITIVE,
+         bool     _nullptr_TYPE,
+         typename _UnsignedBits,
+         typename T>
 struct BaseTraits
 {
     /// Category
     static const Category CATEGORY      = _CATEGORY;
     enum
     {
-        PRIMITIVE       = _PRIMITIVE,
-        NULL_TYPE       = _NULL_TYPE,
+        PRIMITIVE    = _PRIMITIVE,
+        nullptr_TYPE = _nullptr_TYPE,
     };
 };
 
@@ -443,7 +529,7 @@ struct BaseTraits
 template <typename _UnsignedBits, typename T>
 struct BaseTraits<UNSIGNED_INTEGER, true, false, _UnsignedBits, T>
 {
-    typedef _UnsignedBits       UnsignedBits;
+    using UnsignedBits = _UnsignedBits;
 
     static const Category       CATEGORY    = UNSIGNED_INTEGER;
     static const UnsignedBits   LOWEST_KEY  = UnsignedBits(0);
@@ -451,11 +537,11 @@ struct BaseTraits<UNSIGNED_INTEGER, true, false, _UnsignedBits, T>
 
     enum
     {
-        PRIMITIVE       = true,
-        NULL_TYPE       = false,
+        PRIMITIVE    = true,
+        nullptr_TYPE = false,
     };
 
-    using key_codec = rocprim::radix_key_codec<T>;
+    using key_codec = decltype(::rocprim::traits::get<T>().template radix_key_codec<false>());
 
     static HIPCUB_HOST_DEVICE __forceinline__ UnsignedBits TwiddleIn(UnsignedBits key)
     {
@@ -491,7 +577,7 @@ struct BaseTraits<UNSIGNED_INTEGER, true, false, _UnsignedBits, T>
 template <typename _UnsignedBits, typename T>
 struct BaseTraits<SIGNED_INTEGER, true, false, _UnsignedBits, T>
 {
-    typedef _UnsignedBits       UnsignedBits;
+    using UnsignedBits = _UnsignedBits;
 
     static const Category       CATEGORY    = SIGNED_INTEGER;
     static const UnsignedBits   HIGH_BIT    = UnsignedBits(1) << ((sizeof(UnsignedBits) * 8) - 1);
@@ -500,11 +586,11 @@ struct BaseTraits<SIGNED_INTEGER, true, false, _UnsignedBits, T>
 
     enum
     {
-        PRIMITIVE       = true,
-        NULL_TYPE       = false,
+        PRIMITIVE    = true,
+        nullptr_TYPE = false,
     };
 
-    using key_codec = rocprim::radix_key_codec<T>;
+    using key_codec = decltype(::rocprim::traits::get<T>().template radix_key_codec<false>());
 
     static HIPCUB_HOST_DEVICE __forceinline__ UnsignedBits TwiddleIn(UnsignedBits key)
     {
@@ -590,19 +676,19 @@ struct FpLimits<hip_bfloat16>
 template <typename _UnsignedBits, typename T>
 struct BaseTraits<FLOATING_POINT, true, false, _UnsignedBits, T>
 {
-    typedef _UnsignedBits       UnsignedBits;
+    using UnsignedBits = _UnsignedBits;
 
     static const Category       CATEGORY    = FLOATING_POINT;
     static const UnsignedBits   HIGH_BIT    = UnsignedBits(1) << ((sizeof(UnsignedBits) * 8) - 1);
     static const UnsignedBits   LOWEST_KEY  = UnsignedBits(-1);
     static const UnsignedBits   MAX_KEY     = UnsignedBits(-1) ^ HIGH_BIT;
 
-    using key_codec = rocprim::radix_key_codec<T>;
+    using key_codec = decltype(::rocprim::traits::get<T>().template radix_key_codec<false>());
 
     enum
     {
-        PRIMITIVE       = true,
-        NULL_TYPE       = false,
+        PRIMITIVE    = true,
+        nullptr_TYPE = false,
     };
 
     static HIPCUB_HOST_DEVICE __forceinline__ UnsignedBits TwiddleIn(UnsignedBits key)
@@ -659,9 +745,9 @@ struct NumericTraits<__uint128_t>
     static constexpr UnsignedBits MAX_KEY    = UnsignedBits(-1);
 
     static constexpr bool PRIMITIVE = false;
-    static constexpr bool NULL_TYPE = false;
+    static constexpr bool nullptr_TYPE = false;
 
-    using key_codec = rocprim::radix_key_codec<T>;
+    using key_codec = decltype(::rocprim::traits::get<T>().template radix_key_codec<false>());
 
     static __host__ __device__ __forceinline__ UnsignedBits TwiddleIn(UnsignedBits key)
     {
@@ -696,9 +782,9 @@ struct NumericTraits<__int128_t>
     static constexpr UnsignedBits MAX_KEY    = UnsignedBits(-1) ^ HIGH_BIT;
 
     static constexpr bool PRIMITIVE = false;
-    static constexpr bool NULL_TYPE = false;
+    static constexpr bool nullptr_TYPE = false;
 
-    using key_codec = rocprim::radix_key_codec<T>;
+    using key_codec = decltype(::rocprim::traits::get<T>().template radix_key_codec<false>());
 
     static __host__ __device__ __forceinline__ UnsignedBits TwiddleIn(UnsignedBits key)
     {
@@ -737,6 +823,125 @@ template <> struct NumericTraits<bool> :                BaseTraits<UNSIGNED_INTE
 template <typename T>
 struct Traits : NumericTraits<typename std::remove_cv<T>::type> {};
 
+/**
+ * \brief Common type of zero types.
+ */
+template<class...>
+struct common_type
+{};
+
+/**
+ * \brief Common type of a single type.
+ */
+template<class T>
+struct common_type<T> : common_type<T, T>
+{};
+
+// Common type of a pair of types.
+namespace detail
+{
+
+// Determines if type is half or bfloat16 (extended fp).
+template<class T>
+struct is_extended_fp
+    : std::integral_constant<
+          bool,
+          std::is_same<__half, typename std::remove_cv<T>::type>::value
+              || std::is_same<hip_bfloat16, typename std::remove_cv<T>::type>::value>
+{};
+
+// Gets "raw" type: drops reference and const qualifier.
+template<typename T>
+struct remove_cvref
+{
+    using type = typename std::remove_cv<typename std::remove_reference<T>::type>::type;
+};
+template<typename T>
+using remove_cvref_t = typename remove_cvref<T>::type;
+
+template<template<typename...> class MFn, bool condition, typename T>
+using apply_if_t = std::conditional_t<condition, MFn<T>, T>;
+
+template<typename From, typename To>
+using copy_cv_t
+    = apply_if_t<std::add_volatile_t,
+                 std::is_volatile_v<From>,
+                 apply_if_t<std::add_const_t, std::is_const_v<From>, std::remove_cv_t<To>>>;
+
+template<typename From, typename To>
+using copy_ref_t = apply_if_t<std::add_rvalue_reference_t,
+                              std::is_rvalue_reference_v<From>,
+                              apply_if_t<std::add_lvalue_reference_t,
+                                         std::is_lvalue_reference_v<From>,
+                                         std::remove_reference_t<To>>>;
+
+template<typename From, typename To>
+using copy_cvref_t = copy_ref_t<From, copy_cv_t<std::remove_reference_t<From>, remove_cvref_t<To>>>;
+
+// Captures non extended fp types.
+template<class T1, class T2, class = void>
+struct common_type_extended_fp
+{
+    using type = typename std::common_type<T1, T2>::type;
+};
+
+// Captures first type arithmetic, second one extended FP.
+template<class T1, class T2>
+struct common_type_extended_fp<
+    T1,
+    T2,
+    typename std::enable_if_t<std::is_arithmetic<remove_cvref_t<T1>>::value
+                              && is_extended_fp<remove_cvref_t<T2>>::value>>
+{
+    using type = typename std::common_type<T1, copy_cvref_t<T2, float>>::type;
+};
+
+// Captures first type extended FP, second one arithmetic.
+template<class T1, class T2>
+struct common_type_extended_fp<
+    T1,
+    T2,
+    typename std::enable_if_t<is_extended_fp<remove_cvref_t<T1>>::value
+                              && std::is_arithmetic<remove_cvref_t<T2>>::value>>
+{
+    using type = typename std::common_type<copy_cvref_t<T1, float>, T2>::type;
+};
+
+template<class...>
+using void_t = void;
+
+// Common type of more than two types.
+
+template<class AlwaysVoid, class T1, class T2, class... Rest>
+struct common_type_multi_impl
+{};
+
+template<class T1, class T2, class... Rest>
+struct common_type_multi_impl<void_t<typename common_type<T1, T2>::type>, T1, T2, Rest...>
+    : common_type<typename common_type<T1, T2>::type, Rest...>
+{};
+} // namespace detail
+
+/**
+ * \brief Common type of a pair of types
+ */
+template<class T1, class T2>
+struct common_type<T1, T2> : detail::common_type_extended_fp<T1, T2>
+{};
+
+/**
+ * \brief Common type of more than two types
+ */
+template<class T1, class T2, class... Rest>
+struct common_type<T1, T2, Rest...> : detail::common_type_multi_impl<void, T1, T2, Rest...>
+{};
+
+/**
+ * \brief Common type helper
+ */
+template<class... Ts>
+using common_type_t = typename common_type<Ts...>::type;
+
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 
 END_HIPCUB_NAMESPACE
