@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -30,6 +30,7 @@
 
 // hipcub API
 #include "hipcub/device/device_histogram.hpp"
+#include "hipcub/iterator/counting_input_iterator.hpp"
 #include "hipcub/iterator/transform_input_iterator.hpp"
 
 // rows, columns, (row_stride - columns * Channels)
@@ -104,22 +105,22 @@ struct transform_op
     }
 };
 
-template<
-    class SampleType,
-    unsigned int Bins,
-    int LowerLevel,
-    int UpperLevel,
-    class LevelType = SampleType,
-    class CounterType = int
->
+template<class SampleType,
+         unsigned int Bins,
+         int          LowerLevel,
+         int          UpperLevel,
+         class LevelType   = SampleType,
+         class CounterType = int,
+         bool UseGraphs    = false>
 struct params1
 {
-    using sample_type = SampleType;
-    static constexpr unsigned int bins = Bins;
-    static constexpr int lower_level = LowerLevel;
-    static constexpr int upper_level = UpperLevel;
-    using level_type = LevelType;
-    using counter_type = CounterType;
+    using sample_type                         = SampleType;
+    static constexpr unsigned int bins        = Bins;
+    static constexpr int          lower_level = LowerLevel;
+    static constexpr int          upper_level = UpperLevel;
+    using level_type                          = LevelType;
+    using counter_type                        = CounterType;
+    static constexpr bool use_graphs          = UseGraphs;
 };
 
 template<class Params>
@@ -128,18 +129,21 @@ public:
     using params = Params;
 };
 
-typedef ::testing::Types<params1<int, 10, 0, 10>,
-                         params1<int, 128, 0, 256>,
-                         params1<unsigned int, 12345, 10, 12355, short>,
-                         params1<unsigned short, 65536, 0, 65536, int>,
-                         params1<unsigned char, 10, 20, 240, unsigned char, unsigned int>,
-                         params1<unsigned char, 256, 0, 256, short>,
-                         params1<test_utils::half, 55, -123, +123, test_utils::half>,
-                         params1<test_utils::bfloat16, 55, -123, +123, test_utils::bfloat16>,
-                         params1<double, 10, 0, 1000, double, int>,
-                         params1<int, 123, 100, 5635, int>,
-                         params1<double, 55, -123, +123, double>>
-    Params1;
+using Params1
+    = ::testing::Types<params1<int, 10, 0, 10>,
+                       params1<int, 128, 0, 256>,
+                       params1<unsigned int, 12345, 10, 12355, short>,
+                       params1<unsigned short, 65536, 0, 65536, int>,
+                       params1<unsigned char, 10, 20, 240, unsigned char, unsigned int>,
+                       params1<unsigned char, 256, 0, 256, short>,
+                       params1<test_utils::half, 55, -123, +123, test_utils::half>,
+                       params1<test_utils::bfloat16, 55, -123, +123, test_utils::bfloat16>,
+                       params1<double, 10, 0, 1000, double, int>,
+                       params1<int, 123, 100, 5635, int>,
+                       params1<double, 55, -123, +123, double>,
+                       params1<int, 10, 0, 10, int, int, true>,
+                       // Regression: sample_type = int and level_type = size_t
+                       params1<int, 123, 100, 5635, size_t>>;
 
 TYPED_TEST_SUITE(HipcubDeviceHistogramEven, Params1);
 
@@ -172,7 +176,12 @@ TYPED_TEST(HipcubDeviceHistogramEven, Even)
         GTEST_SKIP();
     }
 
-    hipStream_t stream = 0;
+    hipStream_t stream = 0; // default
+    if(TestFixture::params::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     for(auto dim : get_dims())
     {
@@ -264,6 +273,10 @@ TYPED_TEST(HipcubDeviceHistogramEven, Even)
             void * d_temporary_storage;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
+            test_utils::GraphHelper gHelper;
+            if (TestFixture::params::use_graphs)
+                gHelper.startStreamCapture(stream);
+
             if(rows == 1)
             {
                 HIP_CHECK(hipcub::DeviceHistogram::HistogramEven(d_temporary_storage,
@@ -291,6 +304,9 @@ TYPED_TEST(HipcubDeviceHistogramEven, Even)
                                                                  stream));
             }
 
+            if (TestFixture::params::use_graphs)
+                gHelper.createAndLaunchGraph(stream);
+
             std::vector<counter_type> histogram(bins);
             HIP_CHECK(
                 hipMemcpy(
@@ -308,28 +324,137 @@ TYPED_TEST(HipcubDeviceHistogramEven, Even)
             {
                 ASSERT_EQ(histogram[i], histogram_expected[i]);
             }
+
+            if(TestFixture::params::use_graphs)
+                gHelper.cleanupGraphHelper();
+        }
+    }
+
+    if(TestFixture::params::use_graphs)
+        HIP_CHECK(hipStreamDestroy(stream));
+}
+
+// Test HistogramEven overflow
+template<class Params>
+class HipcubDeviceHistogramEvenOverflow : public ::testing::Test
+{
+public:
+    using params = Params;
+};
+
+using Params1Overflow = ::testing::Types<params1<uint16_t, 1, 0, 10>,
+                                         params1<uint16_t, 2, 0, 10>,
+                                         params1<uint32_t, 1, 0, 10>,
+                                         params1<uint32_t, 2, 0, 10>,
+                                         params1<uint64_t, 1, 0, 10>,
+                                         params1<uint64_t, 2, 0, 10>,
+                                         params1<uint64_t, 2, 0, 10, float>,
+                                         // Test extended float types
+                                         params1<uint64_t, 2, 0, 10, test_utils::half>,
+                                         params1<uint64_t, 2, 0, 10, test_utils::bfloat16>>;
+
+TYPED_TEST_SUITE(HipcubDeviceHistogramEvenOverflow, Params1Overflow);
+
+TYPED_TEST(HipcubDeviceHistogramEvenOverflow, EvenOverflow)
+{
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
+    using sample_type           = typename TestFixture::params::sample_type;
+    using counter_type          = uint32_t;
+    using level_type            = typename TestFixture::params::level_type;
+    constexpr unsigned int bins = TestFixture::params::bins;
+
+    // native host types
+    using native_level_type = test_utils::convert_to_fundamental_t<level_type>;
+
+    const native_level_type n_lower_level = 0;
+    const native_level_type n_upper_level
+        = static_cast<native_level_type>(std::numeric_limits<sample_type>::max());
+
+    level_type lower_level = test_utils::convert_to_device<level_type>(n_lower_level);
+    level_type upper_level = test_utils::convert_to_device<level_type>(n_upper_level);
+
+    hipStream_t stream = 0; // default
+
+    const size_t size = 1000;
+
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
+        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+
+        // Generate data
+        auto          d_input = hipcub::CountingInputIterator<sample_type>{0UL};
+        counter_type* d_histogram;
+        HIP_CHECK(test_common_utils::hipMallocHelper(&d_histogram, bins * sizeof(counter_type)));
+
+        size_t     temporary_storage_bytes = 0;
+        hipError_t error                   = hipcub::DeviceHistogram::HistogramEven(nullptr,
+                                                                  temporary_storage_bytes,
+                                                                  d_input,
+                                                                  d_histogram,
+                                                                  bins + 1,
+                                                                  lower_level,
+                                                                  upper_level,
+                                                                  int(size),
+                                                                  stream);
+
+        // Allocate a some amount of temp storage bytes in case of an overflow of the bin
+        //  computation. Note that the subsequent algorithm invocation will also fail.
+        if(error == hipErrorInvalidValue)
+        {
+            temporary_storage_bytes = 3;
+        }
+
+        void* d_temporary_storage;
+        HIP_CHECK(
+            test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
+
+        error = hipcub::DeviceHistogram::HistogramEven(d_temporary_storage,
+                                                       temporary_storage_bytes,
+                                                       d_input,
+                                                       d_histogram,
+                                                       bins + 1,
+                                                       lower_level,
+                                                       upper_level,
+                                                       int(size),
+                                                       stream);
+
+        HIP_CHECK(hipFree(d_temporary_storage));
+        HIP_CHECK(hipFree(d_histogram));
+
+        if(bins == 1 || sizeof(sample_type) <= 4UL || !std::is_integral<native_level_type>())
+        {
+            ASSERT_EQ(error, hipSuccess);
+        }
+        else
+        {
+            ASSERT_EQ(error, hipErrorInvalidValue);
         }
     }
 }
 
-template<
-    class SampleType,
-    unsigned int Bins,
-    int StartLevel = 0,
-    unsigned int MinBinWidth = 1,
-    unsigned int MaxBinWidth = 10,
-    class LevelType = SampleType,
-    class CounterType = int
->
+template<class SampleType,
+         unsigned int Bins,
+         int          StartLevel  = 0,
+         unsigned int MinBinWidth = 1,
+         unsigned int MaxBinWidth = 10,
+         class LevelType          = SampleType,
+         class CounterType        = int,
+         bool UseGraphs           = false>
 struct params2
 {
-    using sample_type = SampleType;
-    static constexpr unsigned int bins = Bins;
-    static constexpr int start_level = StartLevel;
+    using sample_type                            = SampleType;
+    static constexpr unsigned int bins           = Bins;
+    static constexpr int          start_level    = StartLevel;
     static constexpr unsigned int min_bin_length = MinBinWidth;
     static constexpr unsigned int max_bin_length = MaxBinWidth;
-    using level_type = LevelType;
-    using counter_type = CounterType;
+    using level_type                             = LevelType;
+    using counter_type                           = CounterType;
+    static constexpr bool use_graphs             = UseGraphs;
 };
 
 template<class Params>
@@ -338,7 +463,7 @@ public:
     using params = Params;
 };
 
-typedef ::testing::Types<
+using Params2 = ::testing::Types<
     params2<int, 10, 0, 1, 10>,
     params2<unsigned char, 5, 10, 10, 20>,
     params2<unsigned int, 10000, 0, 1, 100>,
@@ -347,8 +472,10 @@ typedef ::testing::Types<
     params2<test_utils::half, 3, 10000, 1000, 1000, test_utils::half, unsigned int>,
     params2<test_utils::bfloat16, 3, 10000, 1000, 1000, test_utils::bfloat16, unsigned int>,
     params2<float, 456, -100, 1, 123>,
-    params2<double, 3, 10000, 1000, 1000, double, unsigned int>>
-    Params2;
+    params2<double, 3, 10000, 1000, 1000, double, unsigned int>,
+    params2<int, 10, 0, 1, 10, int, int, true>,
+    // Regression: sample_type = int and level_type = size_t
+    params2<int, 10, 0, 1, 10, size_t>>;
 
 TYPED_TEST_SUITE(HipcubDeviceHistogramRange, Params2);
 
@@ -368,7 +495,12 @@ TYPED_TEST(HipcubDeviceHistogramRange, Range)
     using native_sample_type = test_utils::convert_to_fundamental_t<sample_type>;
     using native_level_type  = test_utils::convert_to_fundamental_t<level_type>;
 
-    hipStream_t stream = 0;
+    hipStream_t stream = 0; // default
+    if(TestFixture::params::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     std::random_device rd;
     std::default_random_engine gen(rd());
@@ -487,6 +619,10 @@ TYPED_TEST(HipcubDeviceHistogramRange, Range)
             void * d_temporary_storage;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
+            test_utils::GraphHelper gHelper;
+            if (TestFixture::params::use_graphs)
+                gHelper.startStreamCapture(stream);
+
             if(rows == 1)
             {
                 HIP_CHECK(hipcub::DeviceHistogram::HistogramRange(d_temporary_storage,
@@ -512,6 +648,9 @@ TYPED_TEST(HipcubDeviceHistogramRange, Range)
                                                                   stream));
             }
 
+            if (TestFixture::params::use_graphs)
+                gHelper.createAndLaunchGraph(stream);
+
             std::vector<counter_type> histogram(bins);
             HIP_CHECK(
                 hipMemcpy(
@@ -530,30 +669,35 @@ TYPED_TEST(HipcubDeviceHistogramRange, Range)
             {
                 ASSERT_EQ(histogram[i], histogram_expected[i]);
             }
+
+            if(TestFixture::params::use_graphs)
+                gHelper.cleanupGraphHelper();
         }
     }
+    if(TestFixture::params::use_graphs)
+        HIP_CHECK(hipStreamDestroy(stream));
 }
 
-template<
-    class SampleType,
-    unsigned int Channels,
-    unsigned int ActiveChannels,
-    unsigned int Bins,
-    int LowerLevel,
-    int UpperLevel,
-    class LevelType = SampleType,
-    class CounterType = int
->
+template<class SampleType,
+         unsigned int Channels,
+         unsigned int ActiveChannels,
+         unsigned int Bins,
+         int          LowerLevel,
+         int          UpperLevel,
+         class LevelType   = SampleType,
+         class CounterType = int,
+         bool UseGraphs    = false>
 struct params3
 {
-    using sample_type = SampleType;
-    static constexpr unsigned int channels = Channels;
+    using sample_type                             = SampleType;
+    static constexpr unsigned int channels        = Channels;
     static constexpr unsigned int active_channels = ActiveChannels;
-    static constexpr unsigned int bins = Bins;
-    static constexpr int lower_level = LowerLevel;
-    static constexpr int upper_level = UpperLevel;
-    using level_type = LevelType;
-    using counter_type = CounterType;
+    static constexpr unsigned int bins            = Bins;
+    static constexpr int          lower_level     = LowerLevel;
+    static constexpr int          upper_level     = UpperLevel;
+    using level_type                              = LevelType;
+    using counter_type                            = CounterType;
+    static constexpr bool use_graphs              = UseGraphs;
 };
 
 template<class Params>
@@ -562,19 +706,22 @@ public:
     using params = Params;
 };
 
-typedef ::testing::Types<params3<int, 4, 3, 2000, 0, 2000>,
-                         params3<int, 2, 1, 10, 0, 10>,
-                         params3<int, 3, 3, 128, 0, 256>,
-                         params3<unsigned int, 1, 1, 12345, 10, 12355, short>,
-                         params3<unsigned int, 4, 4, 65536, 0, 65536, int>,
-                         params3<unsigned char, 3, 1, 10, 20, 240, unsigned char, unsigned int>,
-                         params3<unsigned char, 2, 2, 256, 0, 256, short>,
-                         params3<test_utils::half, 4, 3, 55, -123, +123, test_utils::half>,
-                         params3<test_utils::bfloat16, 4, 3, 55, -123, +123, test_utils::bfloat16>,
-                         params3<double, 4, 2, 10, 0, 1000, double, int>,
-                         params3<int, 3, 2, 123, 100, 5635, int>,
-                         params3<double, 4, 3, 55, -123, +123, double>>
-    Params3;
+using Params3
+    = ::testing::Types<params3<int, 4, 3, 2000, 0, 2000>,
+                       params3<int, 2, 1, 10, 0, 10>,
+                       params3<int, 3, 3, 128, 0, 256>,
+                       params3<unsigned int, 1, 1, 12345, 10, 12355, short>,
+                       params3<unsigned int, 4, 4, 65536, 0, 65536, int>,
+                       params3<unsigned char, 3, 1, 10, 20, 240, unsigned char, unsigned int>,
+                       params3<unsigned char, 2, 2, 256, 0, 256, short>,
+                       params3<test_utils::half, 4, 3, 55, -123, +123, test_utils::half>,
+                       params3<test_utils::bfloat16, 4, 3, 55, -123, +123, test_utils::bfloat16>,
+                       params3<double, 4, 2, 10, 0, 1000, double, int>,
+                       params3<int, 3, 2, 123, 100, 5635, int>,
+                       params3<double, 4, 3, 55, -123, +123, double>,
+                       params3<int, 4, 3, 2000, 0, 2000, int, int, true>,
+                       // Regression: sample_type = int and level_type = size_t
+                       params3<int, 4, 3, 2000, 0, 2000, size_t>>;
 
 TYPED_TEST_SUITE(HipcubDeviceHistogramMultiEven, Params3);
 
@@ -621,7 +768,12 @@ TYPED_TEST(HipcubDeviceHistogramMultiEven, MultiEven)
         upper_level[channel] = test_utils::convert_to_device<level_type>(n_upper_level[channel]);
     }
 
-    hipStream_t stream = 0;
+    hipStream_t stream = 0; // default
+    if(TestFixture::params::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     for(auto dim : get_dims())
     {
@@ -769,6 +921,10 @@ TYPED_TEST(HipcubDeviceHistogramMultiEven, MultiEven)
             void * d_temporary_storage;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
+            test_utils::GraphHelper gHelper;
+            if (TestFixture::params::use_graphs)
+                gHelper.startStreamCapture(stream);
+
             if(rows == 1)
             {
                 HIP_CHECK((hipcub::DeviceHistogram::MultiHistogramEven<channels, active_channels>(
@@ -798,6 +954,9 @@ TYPED_TEST(HipcubDeviceHistogramMultiEven, MultiEven)
                     stream)));
             }
 
+            if (TestFixture::params::use_graphs)
+                gHelper.createAndLaunchGraph(stream);
+
             std::vector<counter_type> histogram[active_channels];
             for(unsigned int channel = 0; channel < active_channels; channel++)
             {
@@ -824,32 +983,38 @@ TYPED_TEST(HipcubDeviceHistogramMultiEven, MultiEven)
                     ASSERT_EQ(histogram[channel][i], histogram_expected[channel][i]);
                 }
             }
+
+            if(TestFixture::params::use_graphs)
+                gHelper.cleanupGraphHelper();
         }
     }
+
+    if(TestFixture::params::use_graphs)
+        HIP_CHECK(hipStreamDestroy(stream));
 }
 
-template<
-    class SampleType,
-    unsigned int Channels,
-    unsigned int ActiveChannels,
-    unsigned int Bins,
-    int StartLevel = 0,
-    unsigned int MinBinWidth = 1,
-    unsigned int MaxBinWidth = 10,
-    class LevelType = SampleType,
-    class CounterType = int
->
+template<class SampleType,
+         unsigned int Channels,
+         unsigned int ActiveChannels,
+         unsigned int Bins,
+         int          StartLevel  = 0,
+         unsigned int MinBinWidth = 1,
+         unsigned int MaxBinWidth = 10,
+         class LevelType          = SampleType,
+         class CounterType        = int,
+         bool UseGraphs           = false>
 struct params4
 {
-    using sample_type = SampleType;
-    static constexpr unsigned int channels = Channels;
+    using sample_type                             = SampleType;
+    static constexpr unsigned int channels        = Channels;
     static constexpr unsigned int active_channels = ActiveChannels;
-    static constexpr unsigned int bins = Bins;
-    static constexpr int start_level = StartLevel;
-    static constexpr unsigned int min_bin_length = MinBinWidth;
-    static constexpr unsigned int max_bin_length = MaxBinWidth;
-    using level_type = LevelType;
-    using counter_type = CounterType;
+    static constexpr unsigned int bins            = Bins;
+    static constexpr int          start_level     = StartLevel;
+    static constexpr unsigned int min_bin_length  = MinBinWidth;
+    static constexpr unsigned int max_bin_length  = MaxBinWidth;
+    using level_type                              = LevelType;
+    using counter_type                            = CounterType;
+    static constexpr bool use_graphs              = UseGraphs;
 };
 
 template<class Params>
@@ -858,7 +1023,7 @@ public:
     using params = Params;
 };
 
-typedef ::testing::Types<
+using Params4 = ::testing::Types<
     params4<int, 4, 3, 10, 0, 1, 10>,
     params4<unsigned char, 2, 2, 5, 10, 10, 20>,
     params4<unsigned int, 1, 1, 10000, 0, 1, 100>,
@@ -867,8 +1032,10 @@ typedef ::testing::Types<
     params4<test_utils::half, 3, 1, 3, 10000, 1000, 1000, test_utils::half, unsigned int>,
     params4<test_utils::bfloat16, 3, 1, 3, 10000, 1000, 1000, test_utils::bfloat16, unsigned int>,
     params4<float, 4, 2, 456, -100, 1, 123>,
-    params4<double, 3, 1, 3, 10000, 1000, 1000, double, unsigned int>>
-    Params4;
+    params4<double, 3, 1, 3, 10000, 1000, 1000, double, unsigned int>,
+    params4<int, 4, 3, 10, 0, 1, 10, int, int, true>,
+    // Regression: sample_type = int and level_type = size_t
+    params4<int, 4, 3, 10, 0, 1, 10, size_t>>;
 
 TYPED_TEST_SUITE(HipcubDeviceHistogramMultiRange, Params4);
 
@@ -890,8 +1057,6 @@ TYPED_TEST(HipcubDeviceHistogramMultiRange, MultiRange)
     constexpr unsigned int channels = TestFixture::params::channels;
     constexpr unsigned int active_channels = TestFixture::params::active_channels;
 
-    hipStream_t stream = 0;
-
     std::random_device rd;
     std::default_random_engine gen(rd());
 
@@ -909,6 +1074,13 @@ TYPED_TEST(HipcubDeviceHistogramMultiRange, MultiRange)
         );
     }
 
+    hipStream_t stream = 0; // default
+    if(TestFixture::params::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
+
     for(auto dim : get_dims())
     {
         SCOPED_TRACE(
@@ -1081,6 +1253,10 @@ TYPED_TEST(HipcubDeviceHistogramMultiRange, MultiRange)
             void * d_temporary_storage;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
+            test_utils::GraphHelper gHelper;
+            if (TestFixture::params::use_graphs)
+                gHelper.startStreamCapture(stream);
+
             if(rows == 1)
             {
                 HIP_CHECK((hipcub::DeviceHistogram::MultiHistogramRange<channels, active_channels>(
@@ -1108,6 +1284,9 @@ TYPED_TEST(HipcubDeviceHistogramMultiRange, MultiRange)
                     stream)));
             }
 
+            if (TestFixture::params::use_graphs)
+                gHelper.createAndLaunchGraph(stream);
+
             std::vector<counter_type> histogram[active_channels];
             for(unsigned int channel = 0; channel < active_channels; channel++)
             {
@@ -1135,6 +1314,12 @@ TYPED_TEST(HipcubDeviceHistogramMultiRange, MultiRange)
                     ASSERT_EQ(histogram[channel][i], histogram_expected[channel][i]);
                 }
             }
+
+            if(TestFixture::params::use_graphs)
+                gHelper.cleanupGraphHelper();
         }
     }
+
+    if(TestFixture::params::use_graphs)
+        HIP_CHECK(hipStreamDestroy(stream));
 }
