@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -26,57 +26,45 @@
 #include "hipcub/device/device_partition.hpp"
 #include "identity_iterator.hpp"
 
+#include "test_utils_data_generation.hpp"
+
 #include <algorithm>
 #include <array>
 
 // Params for tests
-template<
-    class InputType,
-    class OutputType = InputType,
-    class FlagType = unsigned int,
-    bool UseIdentityIterator = false
->
+template<class InputType,
+         class OutputType         = InputType,
+         class FlagType           = unsigned int,
+         bool UseIdentityIterator = false,
+         bool UseGraphs           = false>
 struct DevicePartitionParams
 {
-    using input_type = InputType;
-    using output_type = OutputType;
-    using flag_type = FlagType;
+    using input_type                            = InputType;
+    using output_type                           = OutputType;
+    using flag_type                             = FlagType;
     static constexpr bool use_identity_iterator = UseIdentityIterator;
+    static constexpr bool use_graphs            = UseGraphs;
 };
 
 template<class Params>
 class HipcubDevicePartitionTests : public ::testing::Test
 {
 public:
-    using input_type = typename Params::input_type;
-    using output_type = typename Params::output_type;
+    using input_type                            = typename Params::input_type;
+    using output_type                           = typename Params::output_type;
     using flag_type                             = typename Params::flag_type;
     static constexpr bool use_identity_iterator = Params::use_identity_iterator;
+    static constexpr bool use_graphs            = Params::use_graphs;
 };
 
-typedef ::testing::Types<
-    DevicePartitionParams<int, int, unsigned char, true>,
-    DevicePartitionParams<unsigned int, unsigned long>,
-    DevicePartitionParams<unsigned char, float>,
-    DevicePartitionParams<int8_t, int8_t>,
-    DevicePartitionParams<uint8_t, uint8_t>,
-    DevicePartitionParams<test_utils::custom_test_type<long long>>
-> HipcubDevicePartitionTestsParams;
-
-std::vector<size_t> get_sizes(int seed_value)
-{
-    std::vector<size_t> sizes = {
-        2, 32, 64, 256,
-        1024, 2048,
-        3072, 4096,
-        27845, (1 << 18) + 1111,
-        1024 * 1024 * 32
-    };
-    const std::vector<size_t> random_sizes = test_utils::get_random_data<size_t>(2, 1, 16384, seed_value);
-    sizes.insert(sizes.end(), random_sizes.begin(), random_sizes.end());
-    std::sort(sizes.begin(), sizes.end());
-    return sizes;
-}
+using HipcubDevicePartitionTestsParams
+    = ::testing::Types<DevicePartitionParams<int, int, unsigned char, true>,
+                       DevicePartitionParams<unsigned int, unsigned long>,
+                       DevicePartitionParams<unsigned char, float>,
+                       DevicePartitionParams<int8_t, int8_t>,
+                       DevicePartitionParams<uint8_t, uint8_t>,
+                       DevicePartitionParams<test_utils::custom_test_type<long long>>,
+                       DevicePartitionParams<int, int, unsigned char, false, true>>;
 
 TYPED_TEST_SUITE(HipcubDevicePartitionTests, HipcubDevicePartitionTestsParams);
 
@@ -86,56 +74,50 @@ TYPED_TEST(HipcubDevicePartitionTests, Flagged)
     SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
-    using T = typename TestFixture::input_type;
-    using U = typename TestFixture::output_type;
-    using F = typename TestFixture::flag_type;
+    using T                                     = typename TestFixture::input_type;
+    using U                                     = typename TestFixture::output_type;
+    using F                                     = typename TestFixture::flag_type;
     static constexpr bool use_identity_iterator = TestFixture::use_identity_iterator;
 
-    hipStream_t stream = 0; // default stream
-
-    for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    hipStream_t stream = 0; // default
+    if(TestFixture::use_graphs)
     {
-        unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
+
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
         SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
 
-        const std::vector<size_t> sizes = get_sizes(seed_value);
-        for(auto size : sizes)
+        for(size_t size : test_utils::get_sizes(seed_value))
         {
-            SCOPED_TRACE(testing::Message() << "with size = " << size);
-
+            SCOPED_TRACE(testing::Message() << "with size= " << size);
             // Generate data
             std::vector<T> input = test_utils::get_random_data<T>(size, 1, 100, seed_value);
             std::vector<F> flags = test_utils::get_random_data01<F>(size, 0.25, seed_value);
 
-            T * d_input;
-            F * d_flags;
-            U * d_output;
-            unsigned int * d_selected_count_output;
+            T*            d_input;
+            F*            d_flags;
+            U*            d_output;
+            unsigned int* d_selected_count_output;
             HIP_CHECK(hipMalloc(&d_input, input.size() * sizeof(T)));
             HIP_CHECK(hipMalloc(&d_flags, flags.size() * sizeof(F)));
             HIP_CHECK(hipMalloc(&d_output, input.size() * sizeof(U)));
             HIP_CHECK(hipMalloc(&d_selected_count_output, sizeof(unsigned int)));
             HIP_CHECK(
-                hipMemcpy(
-                    d_input, input.data(),
-                    input.size() * sizeof(T),
-                    hipMemcpyHostToDevice
-                )
-            );
+                hipMemcpy(d_input, input.data(), input.size() * sizeof(T), hipMemcpyHostToDevice));
             HIP_CHECK(
-                hipMemcpy(
-                    d_flags, flags.data(),
-                    flags.size() * sizeof(F),
-                    hipMemcpyHostToDevice
-                )
-            );
+                hipMemcpy(d_flags, flags.data(), flags.size() * sizeof(F), hipMemcpyHostToDevice));
             HIP_CHECK(hipDeviceSynchronize());
 
             // Calculate expected_selected and expected_rejected results on host
             std::vector<U> expected_selected;
             std::vector<U> expected_rejected;
-            expected_selected.reserve(input.size()/2);
-            expected_rejected.reserve(input.size()/2);
+            expected_selected.reserve(input.size() / 2);
+            expected_rejected.reserve(input.size() / 2);
             for(size_t i = 0; i < input.size(); i++)
             {
                 if(flags[i] != 0)
@@ -167,10 +149,14 @@ TYPED_TEST(HipcubDevicePartitionTests, Flagged)
             ASSERT_GT(temp_storage_size_bytes, 0);
 
             // allocate temporary storage
-            void * d_temp_storage = nullptr;
+            void* d_temp_storage = nullptr;
             HIP_CHECK(hipMalloc(&d_temp_storage, temp_storage_size_bytes));
             HIP_CHECK(hipDeviceSynchronize());
 
+            test_utils::GraphHelper gHelper;
+            if (TestFixture::use_graphs)
+                gHelper.startStreamCapture(stream);
+
             // Run
             HIP_CHECK(hipcub::DevicePartition::Flagged(
                 d_temp_storage,
@@ -181,29 +167,27 @@ TYPED_TEST(HipcubDevicePartitionTests, Flagged)
                 d_selected_count_output,
                 input.size(),
                 stream));
+
+            if (TestFixture::use_graphs)
+                gHelper.createAndLaunchGraph(stream);
+
             HIP_CHECK(hipDeviceSynchronize());
 
             // Check if number of selected value is as expected_selected
             unsigned int selected_count_output = 0;
-            HIP_CHECK(
-                hipMemcpy(
-                    &selected_count_output, d_selected_count_output,
-                    sizeof(unsigned int),
-                    hipMemcpyDeviceToHost
-                )
-            );
+            HIP_CHECK(hipMemcpy(&selected_count_output,
+                                d_selected_count_output,
+                                sizeof(unsigned int),
+                                hipMemcpyDeviceToHost));
             HIP_CHECK(hipDeviceSynchronize());
             ASSERT_EQ(selected_count_output, expected_selected.size());
 
             // Check if output values are as expected_selected
             std::vector<U> output(input.size());
-            HIP_CHECK(
-                hipMemcpy(
-                    output.data(), d_output,
-                    output.size() * sizeof(U),
-                    hipMemcpyDeviceToHost
-                )
-            );
+            HIP_CHECK(hipMemcpy(output.data(),
+                                d_output,
+                                output.size() * sizeof(U),
+                                hipMemcpyDeviceToHost));
             HIP_CHECK(hipDeviceSynchronize());
 
             std::vector<U> output_rejected;
@@ -212,16 +196,25 @@ TYPED_TEST(HipcubDevicePartitionTests, Flagged)
                 auto j = i + expected_selected.size();
                 output_rejected.push_back(output[j]);
             }
-            ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected_selected, expected_selected.size()));
-            ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output_rejected, expected_rejected, expected_rejected.size()));
+            ASSERT_NO_FATAL_FAILURE(
+                test_utils::assert_eq(output, expected_selected, expected_selected.size()));
+            ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output_rejected,
+                                                          expected_rejected,
+                                                          expected_rejected.size()));
 
-            hipFree(d_input);
-            hipFree(d_flags);
-            hipFree(d_output);
-            hipFree(d_selected_count_output);
-            hipFree(d_temp_storage);
+            if(TestFixture::use_graphs)
+                gHelper.cleanupGraphHelper();
+
+            HIP_CHECK(hipFree(d_input));
+            HIP_CHECK(hipFree(d_flags));
+            HIP_CHECK(hipFree(d_output));
+            HIP_CHECK(hipFree(d_selected_count_output));
+            HIP_CHECK(hipFree(d_temp_storage));
         }
     }
+
+    if(TestFixture::use_graphs)
+        HIP_CHECK(hipStreamDestroy(stream));
 }
 
 // NOTE: The following lambdas cannot be inside the test because of nvcc
@@ -232,7 +225,8 @@ struct TestSelectOp
     __host__ __device__
     bool operator()(const T& value) const
     {
-        if(value == T(50)) return true;
+        if(value == T(50))
+            return true;
         return false;
     }
 };
@@ -243,55 +237,55 @@ TYPED_TEST(HipcubDevicePartitionTests, If)
     SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
-    using T = typename TestFixture::input_type;
-    using U = typename TestFixture::output_type;
+    using T                                     = typename TestFixture::input_type;
+    using U                                     = typename TestFixture::output_type;
     static constexpr bool use_identity_iterator = TestFixture::use_identity_iterator;
 
-    hipStream_t stream = 0; // default stream
-
 #ifdef __HIP_PLATFORM_NVIDIA__
     TestSelectOp select_op;
 #else
-    auto select_op = [] __host__ __device__ (const T& value) -> bool
+    auto select_op = [](const T& value) -> bool
     {
-        if(value == T(50)) return true;
+        if(value == T(50))
+            return true;
         return false;
     };
 #endif
 
-    for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    hipStream_t stream = 0; // default
+    if(TestFixture::use_graphs)
     {
-        unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
+
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
         SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
 
-        const std::vector<size_t> sizes = get_sizes(seed_value);
-        for(auto size : sizes)
+        for(size_t size : test_utils::get_sizes(seed_value))
         {
-            SCOPED_TRACE(testing::Message() << "with size = " << size);
-
+            SCOPED_TRACE(testing::Message() << "with size= " << size);
             // Generate data
             std::vector<T> input = test_utils::get_random_data<T>(size, 1, 100, seed_value);
 
-            T * d_input;
-            U * d_output;
-            unsigned int * d_selected_count_output;
+            T*            d_input;
+            U*            d_output;
+            unsigned int* d_selected_count_output;
             HIP_CHECK(hipMalloc(&d_input, input.size() * sizeof(T)));
             HIP_CHECK(hipMalloc(&d_output, input.size() * sizeof(U)));
             HIP_CHECK(hipMalloc(&d_selected_count_output, sizeof(unsigned int)));
             HIP_CHECK(
-                hipMemcpy(
-                    d_input, input.data(),
-                    input.size() * sizeof(T),
-                    hipMemcpyHostToDevice
-                )
-            );
+                hipMemcpy(d_input, input.data(), input.size() * sizeof(T), hipMemcpyHostToDevice));
             HIP_CHECK(hipDeviceSynchronize());
 
             // Calculate expected_selected and expected_rejected results on host
             std::vector<U> expected_selected;
             std::vector<U> expected_rejected;
-            expected_selected.reserve(input.size()/2);
-            expected_rejected.reserve(input.size()/2);
+            expected_selected.reserve(input.size() / 2);
+            expected_rejected.reserve(input.size() / 2);
             for(size_t i = 0; i < input.size(); i++)
             {
                 if(select_op(input[i]))
@@ -323,10 +317,14 @@ TYPED_TEST(HipcubDevicePartitionTests, If)
             ASSERT_GT(temp_storage_size_bytes, 0);
 
             // allocate temporary storage
-            void * d_temp_storage = nullptr;
+            void* d_temp_storage = nullptr;
             HIP_CHECK(hipMalloc(&d_temp_storage, temp_storage_size_bytes));
             HIP_CHECK(hipDeviceSynchronize());
 
+            test_utils::GraphHelper gHelper;
+            if (TestFixture::use_graphs)
+                gHelper.startStreamCapture(stream);
+
             // Run
             HIP_CHECK(hipcub::DevicePartition::If(
                 d_temp_storage,
@@ -337,29 +335,27 @@ TYPED_TEST(HipcubDevicePartitionTests, If)
                 input.size(),
                 select_op,
                 stream));
+
+            if (TestFixture::use_graphs)
+                gHelper.createAndLaunchGraph(stream);
+
             HIP_CHECK(hipDeviceSynchronize());
 
             // Check if number of selected value is as expected_selected
             unsigned int selected_count_output = 0;
-            HIP_CHECK(
-                hipMemcpy(
-                    &selected_count_output, d_selected_count_output,
-                    sizeof(unsigned int),
-                    hipMemcpyDeviceToHost
-                )
-            );
+            HIP_CHECK(hipMemcpy(&selected_count_output,
+                                d_selected_count_output,
+                                sizeof(unsigned int),
+                                hipMemcpyDeviceToHost));
             HIP_CHECK(hipDeviceSynchronize());
             ASSERT_EQ(selected_count_output, expected_selected.size());
 
             // Check if output values are as expected_selected
             std::vector<U> output(input.size());
-            HIP_CHECK(
-                hipMemcpy(
-                    output.data(), d_output,
-                    output.size() * sizeof(U),
-                    hipMemcpyDeviceToHost
-                )
-            );
+            HIP_CHECK(hipMemcpy(output.data(),
+                                d_output,
+                                output.size() * sizeof(U),
+                                hipMemcpyDeviceToHost));
             HIP_CHECK(hipDeviceSynchronize());
 
             std::vector<U> output_rejected;
@@ -368,32 +364,43 @@ TYPED_TEST(HipcubDevicePartitionTests, If)
                 auto j = i + expected_selected.size();
                 output_rejected.push_back(output[j]);
             }
-            ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected_selected, expected_selected.size()));
-            ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output_rejected, expected_rejected, expected_rejected.size()));
+            ASSERT_NO_FATAL_FAILURE(
+                test_utils::assert_eq(output, expected_selected, expected_selected.size()));
+            ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output_rejected,
+                                                          expected_rejected,
+                                                          expected_rejected.size()));
 
-            hipFree(d_input);
-            hipFree(d_output);
-            hipFree(d_selected_count_output);
-            hipFree(d_temp_storage);
+            if(TestFixture::use_graphs)
+                gHelper.cleanupGraphHelper();
+
+            HIP_CHECK(hipFree(d_input));
+            HIP_CHECK(hipFree(d_output));
+            HIP_CHECK(hipFree(d_selected_count_output));
+            HIP_CHECK(hipFree(d_temp_storage));
         }
     }
+
+    if(TestFixture::use_graphs)
+        HIP_CHECK(hipStreamDestroy(stream));
 }
 
-namespace {
-template <typename T>
-struct LessOp {
-    HIPCUB_HOST_DEVICE LessOp(const T& pivot)
-        : pivot_{pivot}
+namespace
+{
+template<typename T>
+struct LessOp
+{
+    HIPCUB_HOST_DEVICE LessOp(const T& pivot) : pivot_{pivot} {}
+
+    HIPCUB_HOST_DEVICE
+    bool operator()(const T& val) const
     {
+        return val < pivot_;
     }
 
-    HIPCUB_HOST_DEVICE bool operator()(const T& val) const {
-        return val < pivot_; 
-    }
 private:
     T pivot_;
 };
-}
+} // namespace
 
 TYPED_TEST(HipcubDevicePartitionTests, IfThreeWay)
 {
@@ -401,66 +408,65 @@ TYPED_TEST(HipcubDevicePartitionTests, IfThreeWay)
     SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
-    using T = typename TestFixture::input_type;
-    using U = typename TestFixture::output_type;
+    using T                                     = typename TestFixture::input_type;
+    using U                                     = typename TestFixture::output_type;
     static constexpr bool use_identity_iterator = TestFixture::use_identity_iterator;
 
-    const hipStream_t stream = 0; // default stream
-
-    for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    hipStream_t stream = 0; // default
+    if(TestFixture::use_graphs)
     {
-        const unsigned int seed_value = seed_index < random_seeds_count 
-            ? static_cast<unsigned int>(rand()) : seeds[seed_index - random_seeds_count];
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
+
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
         SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
 
-        const std::vector<size_t> sizes = get_sizes(seed_value);
-        for(auto size : sizes)
+        for(size_t size : test_utils::get_sizes(seed_value))
         {
-            SCOPED_TRACE(testing::Message() << "with size = " << size);
-
+            SCOPED_TRACE(testing::Message() << "with size= " << size);
             // Generate data
             const auto input = test_utils::get_random_data<T>(size, 1, 100, seed_value);
 
             // Outputs
-            auto expected_first_output = std::vector<U>(input.size());
-            auto expected_second_output = std::vector<U>(input.size());
+            auto expected_first_output      = std::vector<U>(input.size());
+            auto expected_second_output     = std::vector<U>(input.size());
             auto expected_unselected_output = std::vector<U>(input.size());
-            auto selected_counts = std::array<unsigned int, 2>{};
+            auto selected_counts            = std::array<unsigned int, 2>{};
 
-            T* d_input                      = nullptr;
-            U* d_first_output               = nullptr;
-            U* d_second_output              = nullptr;
-            U* d_unselected_output          = nullptr;
-            unsigned int* d_selected_counts = nullptr;
+            T*            d_input             = nullptr;
+            U*            d_first_output      = nullptr;
+            U*            d_second_output     = nullptr;
+            U*            d_unselected_output = nullptr;
+            unsigned int* d_selected_counts   = nullptr;
 
             HIP_CHECK(hipMalloc(&d_input, input.size() * sizeof(input[0])));
             HIP_CHECK(hipMalloc(&d_first_output, input.size() * sizeof(U)));
             HIP_CHECK(hipMalloc(&d_second_output, input.size() * sizeof(U)));
             HIP_CHECK(hipMalloc(&d_unselected_output, input.size() * sizeof(U)));
             HIP_CHECK(hipMalloc(&d_selected_counts, sizeof(selected_counts)));
-            HIP_CHECK(
-                hipMemcpy(
-                    d_input, input.data(),
-                    input.size() * sizeof(input[0]),
-                    hipMemcpyHostToDevice
-                )
-            );
+            HIP_CHECK(hipMemcpy(d_input,
+                                input.data(),
+                                input.size() * sizeof(input[0]),
+                                hipMemcpyHostToDevice));
 
-            const auto first_op = LessOp<T>{static_cast<T>(30)};
+            const auto first_op  = LessOp<T>{static_cast<T>(30)};
             const auto second_op = LessOp<T>{static_cast<T>(60)};
 
-            auto copy = input;
-            const auto partion_point = 
-                std::stable_partition(copy.begin(), copy.end(), first_op);
-            const auto second_partiton_point =
-                std::stable_partition(partion_point, copy.end(), second_op);
+            auto       copy          = input;
+            const auto partion_point = std::stable_partition(copy.begin(), copy.end(), first_op);
+            const auto second_partiton_point
+                = std::stable_partition(partion_point, copy.end(), second_op);
 
             const auto expected_counts = std::array<unsigned int, 2>{
                 static_cast<unsigned int>(partion_point - copy.begin()),
-                static_cast<unsigned int>(second_partiton_point - partion_point)
-            };
+                static_cast<unsigned int>(second_partiton_point - partion_point)};
 
-            const auto expected = [&]{
+            const auto expected = [&]
+            {
                 auto result = std::vector<U>(copy.size());
                 std::copy(copy.cbegin(), copy.cend(), result.begin());
                 return result;
@@ -489,6 +495,10 @@ TYPED_TEST(HipcubDevicePartitionTests, IfThreeWay)
             void* d_temp_storage = nullptr;
             HIP_CHECK(hipMalloc(&d_temp_storage, temp_storage_size_bytes));
 
+            test_utils::GraphHelper gHelper;
+            if (TestFixture::use_graphs)
+                gHelper.startStreamCapture(stream);
+
             // Run
             HIP_CHECK(hipcub::DevicePartition::If(
                 d_temp_storage,
@@ -502,54 +512,53 @@ TYPED_TEST(HipcubDevicePartitionTests, IfThreeWay)
                 first_op,
                 second_op,
                 stream));
+
+            if (TestFixture::use_graphs)
+                gHelper.createAndLaunchGraph(stream);
+
             HIP_CHECK(hipDeviceSynchronize());
 
             // Check if number of selected value is as expected_selected
-            HIP_CHECK(
-                hipMemcpy(
-                    selected_counts.data(), d_selected_counts,
-                    sizeof(selected_counts),
-                    hipMemcpyDeviceToHost
-                )
-            );
+            HIP_CHECK(hipMemcpy(selected_counts.data(),
+                                d_selected_counts,
+                                sizeof(selected_counts),
+                                hipMemcpyDeviceToHost));
             ASSERT_EQ(selected_counts, expected_counts);
 
             // Check if output values are as expected_selected
-            const auto output = [&]{
+            const auto output = [&]
+            {
                 auto result = std::vector<U>(input.size());
-                HIP_CHECK(
-                    hipMemcpy(
-                        result.data(), d_first_output,
-                        expected_counts[0] * sizeof(result[0]),
-                        hipMemcpyDeviceToHost
-                    )
-                );
-                HIP_CHECK(
-                    hipMemcpy(
-                        result.data() + expected_counts[0], d_second_output,
-                        expected_counts[1] * sizeof(result[0]),
-                        hipMemcpyDeviceToHost
-                    )
-                );
-                HIP_CHECK(
-                    hipMemcpy(
-                        result.data() + expected_counts[0] + expected_counts[1],
-                        d_unselected_output,
-                        (input.size() - expected_counts[0] - expected_counts[1]) * sizeof(result[0]),
-                        hipMemcpyDeviceToHost
-                    )
-                );
+                HIP_CHECK(hipMemcpy(result.data(),
+                                    d_first_output,
+                                    expected_counts[0] * sizeof(result[0]),
+                                    hipMemcpyDeviceToHost));
+                HIP_CHECK(hipMemcpy(result.data() + expected_counts[0],
+                                    d_second_output,
+                                    expected_counts[1] * sizeof(result[0]),
+                                    hipMemcpyDeviceToHost));
+                HIP_CHECK(hipMemcpy(result.data() + expected_counts[0] + expected_counts[1],
+                                    d_unselected_output,
+                                    (input.size() - expected_counts[0] - expected_counts[1])
+                                        * sizeof(result[0]),
+                                    hipMemcpyDeviceToHost));
                 return result;
             }();
 
-            ASSERT_EQ(output, expected);
+            ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
 
-            hipFree(d_input);
-            hipFree(d_first_output);
-            hipFree(d_second_output);
-            hipFree(d_unselected_output);
-            hipFree(d_selected_counts);
-            hipFree(d_temp_storage);
+            if(TestFixture::use_graphs)
+                gHelper.cleanupGraphHelper();
+
+            HIP_CHECK(hipFree(d_input));
+            HIP_CHECK(hipFree(d_first_output));
+            HIP_CHECK(hipFree(d_second_output));
+            HIP_CHECK(hipFree(d_unselected_output));
+            HIP_CHECK(hipFree(d_selected_counts));
+            HIP_CHECK(hipFree(d_temp_storage));
         }
     }
+
+    if(TestFixture::use_graphs)
+        HIP_CHECK(hipStreamDestroy(stream));
 }
