@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,7 @@
 #include "common_test_header.hpp"
 
 // Thread operators fixes for extended float types
+#include "test_utils_data_generation.hpp"
 #include "test_utils_thread_operators.hpp"
 
 // hipcub API
@@ -32,14 +33,12 @@
 #include <bitset>
 
 // Params for tests
-template<
-    class InputType,
-    class OutputType = InputType
->
+template<class InputType, class OutputType = InputType, bool UseGraphs = false>
 struct DeviceReduceParams
 {
-    using input_type = InputType;
-    using output_type = OutputType;
+    using input_type                 = InputType;
+    using output_type                = OutputType;
+    static constexpr bool use_graphs = UseGraphs;
 };
 
 // ---------------------------------------------------------
@@ -50,11 +49,12 @@ template<class Params>
 class HipcubDeviceReduceTests : public ::testing::Test
 {
 public:
-    using input_type                        = typename Params::input_type;
-    using output_type                       = typename Params::output_type;
+    using input_type                 = typename Params::input_type;
+    using output_type                = typename Params::output_type;
+    static constexpr bool use_graphs = Params::use_graphs;
 };
 
-typedef ::testing::Types<
+using HipcubDeviceReduceTestsParams = ::testing::Types<
     DeviceReduceParams<int, long>,
     DeviceReduceParams<unsigned long>,
     DeviceReduceParams<short>,
@@ -62,7 +62,8 @@ typedef ::testing::Types<
     DeviceReduceParams<short, float>,
     DeviceReduceParams<int, double>,
     DeviceReduceParams<test_utils::half, test_utils::half>,
-    DeviceReduceParams<test_utils::bfloat16, test_utils::bfloat16>
+    DeviceReduceParams<test_utils::bfloat16, test_utils::bfloat16>,
+    DeviceReduceParams<int, long, true>
 #ifdef __HIP_PLATFORM_AMD__
     ,
     DeviceReduceParams<test_utils::half,
@@ -72,21 +73,7 @@ typedef ::testing::Types<
     DeviceReduceParams<test_utils::custom_test_type<float>, test_utils::custom_test_type<float>>,
     DeviceReduceParams<test_utils::custom_test_type<int>, test_utils::custom_test_type<float>>
 #endif
-    >
-    HipcubDeviceReduceTestsParams;
-
-std::vector<size_t> get_sizes()
-{
-    std::vector<size_t> sizes = {
-        1, 10, 53, 211,
-        1024, 2048, 5096,
-        34567, (1 << 17) - 1220
-    };
-    const std::vector<size_t> random_sizes = test_utils::get_random_data<size_t>(2, 1, 16384, rand());
-    sizes.insert(sizes.end(), random_sizes.begin(), random_sizes.end());
-    std::sort(sizes.begin(), sizes.end());
-    return sizes;
-}
+    >;
 
 TYPED_TEST_SUITE(HipcubDeviceReduceTests, HipcubDeviceReduceTestsParams);
 
@@ -99,15 +86,22 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceSum)
     using T = typename TestFixture::input_type;
     using U = typename TestFixture::output_type;
 
-    const std::vector<size_t> sizes = get_sizes();
-    for(auto size : sizes)
+    hipStream_t stream = 0; // default
+    if(TestFixture::use_graphs)
     {
-        for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
-        {
-            unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-            SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
-            SCOPED_TRACE(testing::Message() << "with size = " << size);
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
+        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+
+        for(size_t size : test_utils::get_sizes(seed_value))
+        {
+            SCOPED_TRACE(testing::Message() << "with size= " << size);
             if(test_utils::precision<U>::value * size > 0.5)
             {
                 std::cout << "Test is skipped from size " << size
@@ -117,36 +111,24 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceSum)
                 break;
             }
 
-            hipStream_t stream = 0; // default
-
             // Generate data
-            std::vector<T> input = test_utils::get_random_data<T>(
-                size,
-                1.0f,
-                100.0f,
-                seed_value
-            );
-            std::vector<U> output(1, (U) 0.0f);
+            std::vector<T> input = test_utils::get_random_data<T>(size, 1.0f, 100.0f, seed_value);
+            std::vector<U> output(1, (U)0.0f);
 
-            T * d_input;
-            U * d_output;
+            T* d_input;
+            U* d_output;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_input, input.size() * sizeof(T)));
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_output, output.size() * sizeof(U)));
             HIP_CHECK(
-                hipMemcpy(
-                    d_input, input.data(),
-                    input.size() * sizeof(T),
-                    hipMemcpyHostToDevice
-                )
-            );
+                hipMemcpy(d_input, input.data(), input.size() * sizeof(T), hipMemcpyHostToDevice));
             HIP_CHECK(hipDeviceSynchronize());
 
             // Calculate expected results on host using the same accumulator type than on device
             using Sum =
                 typename AlgebraicSelector<hipcub::Sum, T, U>::type; // For custom_type_test tests
-            using AccumT = hipcub::detail::accumulator_t<Sum, U, T>;
+            using AccumT = hipcub::detail::accumulator_t<Sum, T, U>;
             Sum    sum_op;
-            AccumT tmp_result = U(0.0f); // hipcub::Sum uses as initial type the output type
+            AccumT tmp_result = AccumT(0.0f); // hipcub::Sum uses as initial type the output type
             for(unsigned int i = 0; i < input.size(); i++)
             {
                 tmp_result = sum_op(tmp_result, input[i]);
@@ -155,15 +137,29 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceSum)
 
             // temp storage
             size_t temp_storage_size_bytes;
-            void * d_temp_storage = nullptr;
+            void*  d_temp_storage = nullptr;
             // Get size of d_temp_storage
-            DeviceReduceSelector<T, U> reduce_selector;
-            reduce_selector.reduce_sum(d_temp_storage,
-                                       temp_storage_size_bytes,
-                                       d_input,
-                                       d_output,
-                                       input.size(),
-                                       stream);
+            if constexpr(std::is_same<T, test_utils::half>::value
+                         || std::is_same<T, test_utils::bfloat16>::value)
+            {
+                HIP_CHECK(hipcub::DeviceReduce::Reduce(d_temp_storage,
+                                                       temp_storage_size_bytes,
+                                                       d_input,
+                                                       d_output,
+                                                       input.size(),
+                                                       ExtendedFloatBinOp<hipcub::Sum>(),
+                                                       U(0.f),
+                                                       stream));
+            }
+            else
+            {
+                HIP_CHECK(hipcub::DeviceReduce::Sum(d_temp_storage,
+                                                    temp_storage_size_bytes,
+                                                    d_input,
+                                                    d_output,
+                                                    input.size(),
+                                                    stream));
+            }
 
             // temp_storage_size_bytes must be >0
             ASSERT_GT(temp_storage_size_bytes, 0U);
@@ -172,24 +168,44 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceSum)
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
             HIP_CHECK(hipDeviceSynchronize());
 
+            test_utils::GraphHelper gHelper;
+            if (TestFixture::use_graphs)
+                gHelper.startStreamCapture(stream);
+
             // Run
-            reduce_selector.reduce_sum(d_temp_storage,
-                                       temp_storage_size_bytes,
-                                       d_input,
-                                       d_output,
-                                       input.size(),
-                                       stream);
+            if constexpr(std::is_same<T, test_utils::half>::value
+                         || std::is_same<T, test_utils::bfloat16>::value)
+            {
+                HIP_CHECK(hipcub::DeviceReduce::Reduce(d_temp_storage,
+                                                       temp_storage_size_bytes,
+                                                       d_input,
+                                                       d_output,
+                                                       input.size(),
+                                                       ExtendedFloatBinOp<hipcub::Sum>(),
+                                                       U(0.f),
+                                                       stream));
+            }
+            else
+            {
+                HIP_CHECK(hipcub::DeviceReduce::Sum(d_temp_storage,
+                                                    temp_storage_size_bytes,
+                                                    d_input,
+                                                    d_output,
+                                                    input.size(),
+                                                    stream));
+            }
+
+            if (TestFixture::use_graphs)
+                gHelper.createAndLaunchGraph(stream);
+
             HIP_CHECK(hipPeekAtLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
             // Copy output to host
-            HIP_CHECK(
-                hipMemcpy(
-                    output.data(), d_output,
-                    output.size() * sizeof(U),
-                    hipMemcpyDeviceToHost
-                )
-            );
+            HIP_CHECK(hipMemcpy(output.data(),
+                                d_output,
+                                output.size() * sizeof(U),
+                                hipMemcpyDeviceToHost));
             HIP_CHECK(hipDeviceSynchronize());
 
             // Check if output values are as expected
@@ -198,11 +214,17 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceSum)
                                         expected,
                                         test_utils::precision<U>::value * size));
 
-            hipFree(d_input);
-            hipFree(d_output);
-            hipFree(d_temp_storage);
+            if(TestFixture::use_graphs)
+                gHelper.cleanupGraphHelper();
+
+            HIP_CHECK(hipFree(d_input));
+            HIP_CHECK(hipFree(d_output));
+            HIP_CHECK(hipFree(d_temp_storage));
         }
     }
+
+    if(TestFixture::use_graphs)
+        HIP_CHECK(hipStreamDestroy(stream));
 }
 
 TYPED_TEST(HipcubDeviceReduceTests, ReduceMinimum)
@@ -214,40 +236,41 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceMinimum)
     using T = typename TestFixture::input_type;
     using U = typename TestFixture::output_type;
 
-    const std::vector<size_t> sizes = get_sizes();
-    for(auto size : sizes)
+    hipStream_t stream = 0; // default
+    if(TestFixture::use_graphs)
     {
-        for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
-        {
-            unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-            SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
-            SCOPED_TRACE(testing::Message() << "with size = " << size);
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
-            hipStream_t stream = 0; // default
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
+        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+
+        for(size_t size : test_utils::get_sizes(seed_value))
+        {
+            SCOPED_TRACE(testing::Message() << "with size= " << size);
 
             // Generate data
             std::vector<T> input = test_utils::get_random_data<T>(size, 1.0f, 100.0f, seed_value);
             std::vector<U> output(1, U(0.0f));
 
-            T * d_input;
-            U * d_output;
+            T* d_input;
+            U* d_output;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_input, input.size() * sizeof(T)));
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_output, output.size() * sizeof(U)));
             HIP_CHECK(
-                hipMemcpy(
-                    d_input, input.data(),
-                    input.size() * sizeof(T),
-                    hipMemcpyHostToDevice
-                )
-            );
+                hipMemcpy(d_input, input.data(), input.size() * sizeof(T), hipMemcpyHostToDevice));
             HIP_CHECK(hipDeviceSynchronize());
 
             // Calculate expected results on host using the same accumulator type than on device
             using Min    = typename MinSelector<T, U>::type; // For custom_type_test tests
-            using AccumT = hipcub::detail::accumulator_t<hipcub::Min, U, T>;
+            using AccumT = hipcub::detail::accumulator_t<hipcub::Min, T, U>;
             Min    min_op;
             AccumT tmp_result = test_utils::numeric_limits<
-                T>::max(); // hipcub::Min uses as initial type the input type
+                AccumT>::max(); // hipcub::Min uses as initial type the input type
             for(unsigned int i = 0; i < input.size(); i++)
             {
                 tmp_result = min_op(tmp_result, input[i]);
@@ -256,7 +279,7 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceMinimum)
 
             // temp storage
             size_t temp_storage_size_bytes;
-            void * d_temp_storage = nullptr;
+            void*  d_temp_storage = nullptr;
             // Get size of d_temp_storage
             HIP_CHECK(hipcub::DeviceReduce::Min(d_temp_storage,
                                                 temp_storage_size_bytes,
@@ -272,6 +295,10 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceMinimum)
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
             HIP_CHECK(hipDeviceSynchronize());
 
+            test_utils::GraphHelper gHelper;
+            if (TestFixture::use_graphs)
+                gHelper.startStreamCapture(stream);
+
             // Run
             HIP_CHECK(hipcub::DeviceReduce::Min(d_temp_storage,
                                                 temp_storage_size_bytes,
@@ -279,32 +306,154 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceMinimum)
                                                 d_output,
                                                 input.size(),
                                                 stream));
+
+            if (TestFixture::use_graphs)
+                gHelper.createAndLaunchGraph(stream);
+
             HIP_CHECK(hipPeekAtLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
             // Copy output to host
-            HIP_CHECK(
-                hipMemcpy(
-                    output.data(), d_output,
-                    output.size() * sizeof(U),
-                    hipMemcpyDeviceToHost
-                )
-            );
+            HIP_CHECK(hipMemcpy(output.data(),
+                                d_output,
+                                output.size() * sizeof(U),
+                                hipMemcpyDeviceToHost));
             HIP_CHECK(hipDeviceSynchronize());
 
             // Check if output values are as expected
-            ASSERT_NO_FATAL_FAILURE(test_utils::assert_near(
-                output[0],
-                expected,
-                std::is_same<T, U>::value
-                    ? 0
-                    : std::max(test_utils::precision<T>::value, test_utils::precision<U>::value)));
+            ASSERT_NO_FATAL_FAILURE(
+                test_utils::assert_near(output[0], expected, test_utils::precision<U>::value));
 
-            hipFree(d_input);
-            hipFree(d_output);
-            hipFree(d_temp_storage);
+            if(TestFixture::use_graphs)
+            {
+                gHelper.cleanupGraphHelper();
+            }
+
+            HIP_CHECK(hipFree(d_input));
+            HIP_CHECK(hipFree(d_output));
+            HIP_CHECK(hipFree(d_temp_storage));
         }
     }
+
+    if(TestFixture::use_graphs)
+    {
+        HIP_CHECK(hipStreamDestroy(stream));
+    }
+}
+
+TYPED_TEST(HipcubDeviceReduceTests, ReduceMaximum)
+{
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
+    using T = typename TestFixture::input_type;
+    using U = typename TestFixture::output_type;
+
+    hipStream_t stream = 0; // default
+    if(TestFixture::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
+
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
+        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+
+        for(size_t size : test_utils::get_sizes(seed_value))
+        {
+            SCOPED_TRACE(testing::Message() << "with size= " << size);
+
+            // Generate data
+            std::vector<T> input = test_utils::get_random_data<T>(size, 1.0f, 100.0f, seed_value);
+            std::vector<U> output(1, U(0.0f));
+
+            T* d_input;
+            U* d_output;
+            HIP_CHECK(test_common_utils::hipMallocHelper(&d_input, input.size() * sizeof(T)));
+            HIP_CHECK(test_common_utils::hipMallocHelper(&d_output, output.size() * sizeof(U)));
+            HIP_CHECK(
+                hipMemcpy(d_input, input.data(), input.size() * sizeof(T), hipMemcpyHostToDevice));
+            HIP_CHECK(hipDeviceSynchronize());
+
+            // Calculate expected results on host using the same accumulator type than on device
+            using Max    = typename MaxSelector<T, U>::type; // For custom_type_test tests
+            using AccumT = hipcub::detail::accumulator_t<hipcub::Max, T, U>;
+            Max    max_op;
+            AccumT tmp_result = test_utils::numeric_limits<AccumT>::min();
+            for(unsigned int i = 0; i < input.size(); i++)
+            {
+                tmp_result = max_op(tmp_result, input[i]);
+            }
+            const U expected = static_cast<U>(tmp_result);
+
+            // temp storage
+            size_t temp_storage_size_bytes;
+            void*  d_temp_storage = nullptr;
+            // Get size of d_temp_storage
+            HIP_CHECK(hipcub::DeviceReduce::Max(d_temp_storage,
+                                                temp_storage_size_bytes,
+                                                d_input,
+                                                d_output,
+                                                input.size(),
+                                                stream));
+
+            // temp_storage_size_bytes must be >0
+            ASSERT_GT(temp_storage_size_bytes, 0U);
+
+            // allocate temporary storage
+            HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
+            HIP_CHECK(hipDeviceSynchronize());
+
+            test_utils::GraphHelper gHelper;
+            if(TestFixture::use_graphs)
+            {
+                gHelper.startStreamCapture(stream);
+            }
+
+            // Run
+            HIP_CHECK(hipcub::DeviceReduce::Max(d_temp_storage,
+                                                temp_storage_size_bytes,
+                                                d_input,
+                                                d_output,
+                                                input.size(),
+                                                stream));
+
+            if(TestFixture::use_graphs)
+            {
+                gHelper.createAndLaunchGraph(stream);
+            }
+
+            HIP_CHECK(hipPeekAtLastError());
+            HIP_CHECK(hipDeviceSynchronize());
+
+            // Copy output to host
+            HIP_CHECK(hipMemcpy(output.data(),
+                                d_output,
+                                output.size() * sizeof(U),
+                                hipMemcpyDeviceToHost));
+            HIP_CHECK(hipDeviceSynchronize());
+
+            // Check if output values are as expected
+            ASSERT_NO_FATAL_FAILURE(
+                test_utils::assert_near(output[0], expected, test_utils::precision<U>::value));
+
+            if(TestFixture::use_graphs)
+            {
+                gHelper.cleanupGraphHelper();
+            }
+
+            HIP_CHECK(hipFree(d_input));
+            HIP_CHECK(hipFree(d_output));
+            HIP_CHECK(hipFree(d_temp_storage));
+        }
+    }
+
+    if(TestFixture::use_graphs)
+        HIP_CHECK(hipStreamDestroy(stream));
 }
 
 struct ArgMinDispatch
@@ -356,20 +505,27 @@ void test_argminmax(typename TestFixture::input_type empty_value)
     using Iterator  = typename hipcub::ArgIndexInputIterator<T*, int>;
     using key_value = typename Iterator::value_type;
 
-    DispatchFunction    function;
-    std::vector<size_t> sizes = get_sizes();
-    sizes.push_back(0);
+    DispatchFunction function;
 
-    for(auto size : sizes)
+    hipStream_t stream = 0; // default
+    if(TestFixture::use_graphs)
     {
-        for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
-        {
-            unsigned int seed_value
-                = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
-            SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
-            SCOPED_TRACE(testing::Message() << "with size = " << size);
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
-            hipStream_t stream = 0; // default
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
+        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+
+        std::vector<size_t> sizes = test_utils::get_sizes(seed_value);
+        sizes.push_back(0);
+
+        for(size_t size : sizes)
+        {
+            SCOPED_TRACE(testing::Message() << "with size= " << size);
 
             // Generate data
             std::vector<T>         input = test_utils::get_random_data<T>(size, 0, 200, seed_value);
@@ -413,6 +569,10 @@ void test_argminmax(typename TestFixture::input_type empty_value)
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
             HIP_CHECK(hipDeviceSynchronize());
 
+            test_utils::GraphHelper gHelper;
+            if (TestFixture::use_graphs)
+                gHelper.startStreamCapture(stream);
+
             // Run
             HIP_CHECK(function(d_temp_storage,
                                temp_storage_size_bytes,
@@ -420,6 +580,10 @@ void test_argminmax(typename TestFixture::input_type empty_value)
                                d_output,
                                input.size(),
                                stream));
+
+            if (TestFixture::use_graphs)
+                gHelper.createAndLaunchGraph(stream);
+
             HIP_CHECK(hipPeekAtLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
@@ -437,8 +601,14 @@ void test_argminmax(typename TestFixture::input_type empty_value)
                 test_utils::assert_near(output[0].value,
                                         expected.value,
                                         test_utils::precision<T>::value * size));
+
+            if(TestFixture::use_graphs)
+                gHelper.cleanupGraphHelper();
         }
     }
+
+    if(TestFixture::use_graphs)
+        HIP_CHECK(hipStreamDestroy(stream));
 }
 
 TYPED_TEST(HipcubDeviceReduceTests, ReduceArgMinimum)
@@ -477,7 +647,7 @@ void test_argminmax_allinf(TypeParam value, TypeParam empty_value)
     using Iterator  = typename hipcub::ArgIndexInputIterator<T*, int>;
     using key_value = typename Iterator::value_type;
 
-    hipStream_t      stream            = 0; // default
+    hipStream_t      stream = 0; // default
     DispatchFunction function;
     constexpr size_t size = 100'000;
 
@@ -532,9 +702,7 @@ void test_argminmax_allinf(TypeParam value, TypeParam empty_value)
     }
 }
 
-// TODO: enable for NVIDIA platform once CUB backend incorporates fix
-#ifdef __HIP_PLATFORM_AMD__
-/// ArgMin with all +Inf should result in +Inf.
+// ArgMin with all +Inf should result in +Inf.
 TYPED_TEST(HipcubDeviceReduceArgMinMaxSpecialTests, ReduceArgMinInf)
 {
     test_argminmax_allinf<TypeParam, ArgMinDispatch>(
@@ -542,14 +710,149 @@ TYPED_TEST(HipcubDeviceReduceArgMinMaxSpecialTests, ReduceArgMinInf)
         test_utils::numeric_limits<TypeParam>::max());
 }
 
-/// ArgMax with all -Inf should result in -Inf.
+// ArgMax with all -Inf should result in -Inf.
 TYPED_TEST(HipcubDeviceReduceArgMinMaxSpecialTests, ReduceArgMaxInf)
 {
     test_argminmax_allinf<TypeParam, ArgMaxDispatch>(
         test_utils::numeric_limits<TypeParam>::infinity_neg(),
         test_utils::numeric_limits<TypeParam>::lowest());
 }
-#endif // __HIP_PLATFORM_AMD__
+
+struct TestTransformOp
+{
+    template<class T>
+    HIPCUB_HOST_DEVICE
+    T operator()(const T& x) const
+    {
+        return x + T(5);
+    }
+};
+
+TYPED_TEST(HipcubDeviceReduceTests, TransformReduce)
+{
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
+    using T = typename TestFixture::input_type;
+    using U = typename TestFixture::output_type;
+
+    hipStream_t stream = 0; // default
+    if(TestFixture::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
+
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
+        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+
+        for(size_t size : test_utils::get_sizes(seed_value))
+        {
+            SCOPED_TRACE(testing::Message() << "with size= " << size);
+            if(test_utils::precision<U>::value * size > 0.5)
+            {
+                std::cout << "Test is skipped from size " << size
+                          << " on, potential error of summation is more than 0.5 of the result "
+                             "with current or larger size"
+                          << std::endl;
+                break;
+            }
+
+            // Generate data
+            std::vector<T> input = test_utils::get_random_data<T>(size, 1.0f, 100.0f, seed_value);
+            std::vector<U> output(1, U(0));
+
+            T* d_input;
+            U* d_output;
+            HIP_CHECK(test_common_utils::hipMallocHelper(&d_input, input.size() * sizeof(T)));
+            HIP_CHECK(test_common_utils::hipMallocHelper(&d_output, output.size() * sizeof(U)));
+            HIP_CHECK(
+                hipMemcpy(d_input, input.data(), input.size() * sizeof(T), hipMemcpyHostToDevice));
+
+            // Calculate expected results on host using the same accumulator type than on device
+            using Sum =
+                typename AlgebraicSelector<hipcub::Sum, T, U>::type; // For custom_type_test tests
+            using AccumT = hipcub::detail::accumulator_t<Sum, T, U>;
+
+            Sum             reduction_op;
+            TestTransformOp transform_op;
+            const U         init(10);
+            AccumT          tmp_result = init; // hipcub::Sum uses as initial type the output type
+            for(size_t i = 0; i < input.size(); ++i)
+            {
+                tmp_result = reduction_op(tmp_result, transform_op(input[i]));
+            }
+            const U expected = static_cast<U>(tmp_result);
+
+            // temp storage
+            size_t temp_storage_size_bytes;
+            void*  d_temp_storage = nullptr;
+            // Get size of d_temp_storage
+            HIP_CHECK(hipcub::DeviceReduce::TransformReduce(d_temp_storage,
+                                                            temp_storage_size_bytes,
+                                                            d_input,
+                                                            d_output,
+                                                            input.size(),
+                                                            reduction_op,
+                                                            transform_op,
+                                                            init,
+                                                            stream));
+
+            // temp_storage_size_bytes must be >0
+            ASSERT_GT(temp_storage_size_bytes, 0U);
+
+            // allocate temporary storage
+            HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
+
+            test_utils::GraphHelper gHelper;
+            if (TestFixture::use_graphs)
+                gHelper.startStreamCapture(stream);
+
+            // Run
+            HIP_CHECK(hipcub::DeviceReduce::TransformReduce(d_temp_storage,
+                                                            temp_storage_size_bytes,
+                                                            d_input,
+                                                            d_output,
+                                                            input.size(),
+                                                            reduction_op,
+                                                            transform_op,
+                                                            init,
+                                                            stream));
+
+            if (TestFixture::use_graphs)
+                gHelper.createAndLaunchGraph(stream);
+
+            HIP_CHECK(hipPeekAtLastError());
+            HIP_CHECK(hipDeviceSynchronize());
+
+            // Copy output to host
+            HIP_CHECK(hipMemcpy(output.data(),
+                                d_output,
+                                output.size() * sizeof(U),
+                                hipMemcpyDeviceToHost));
+
+            // Check if output values are as expected
+            ASSERT_NO_FATAL_FAILURE(
+                test_utils::assert_near(output[0],
+                                        expected,
+                                        test_utils::precision<U>::value * size));
+
+            if(TestFixture::use_graphs)
+                gHelper.cleanupGraphHelper();
+
+            HIP_CHECK(hipFree(d_input));
+            HIP_CHECK(hipFree(d_output));
+            HIP_CHECK(hipFree(d_temp_storage));
+        }
+    }
+
+    if(TestFixture::use_graphs)
+        HIP_CHECK(hipStreamDestroy(stream));
+}
 
 // ---------------------------------------------------------
 // Test for large indices
@@ -559,17 +862,17 @@ template<class Params>
 class HipcubDeviceReduceLargeIndicesTests : public ::testing::Test
 {
 public:
-    using input_type                        = typename Params::input_type;
-    using output_type                       = typename Params::output_type;
+    using input_type  = typename Params::input_type;
+    using output_type = typename Params::output_type;
 };
 
-typedef ::testing::Types<DeviceReduceParams<short, size_t>,
-                         DeviceReduceParams<int, size_t>,
-                         DeviceReduceParams<unsigned int, size_t>,
-                         DeviceReduceParams<unsigned long, size_t>,
-                         DeviceReduceParams<float, size_t>,
-                         DeviceReduceParams<double, size_t>>
-    HipcubDeviceReduceLargeIndicesTestsParams;
+using HipcubDeviceReduceLargeIndicesTestsParams
+    = ::testing::Types<DeviceReduceParams<short, size_t>,
+                       DeviceReduceParams<int, size_t>,
+                       DeviceReduceParams<unsigned int, size_t>,
+                       DeviceReduceParams<unsigned long, size_t>,
+                       DeviceReduceParams<float, size_t>,
+                       DeviceReduceParams<double, size_t>>;
 
 TYPED_TEST_SUITE(HipcubDeviceReduceLargeIndicesTests, HipcubDeviceReduceLargeIndicesTestsParams);
 
@@ -579,9 +882,9 @@ TYPED_TEST(HipcubDeviceReduceLargeIndicesTests, LargeIndices)
     SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
-    using T                      = typename TestFixture::input_type;
-    using U                      = typename TestFixture::output_type;
-    using IteratorType           = hipcub::ConstantInputIterator<T>;
+    using T            = typename TestFixture::input_type;
+    using U            = typename TestFixture::output_type;
+    using IteratorType = hipcub::ConstantInputIterator<T>;
 
     const std::vector<size_t> exponents = {30, 31, 32, 33, 34};
     for(auto exponent : exponents)
@@ -592,7 +895,7 @@ TYPED_TEST(HipcubDeviceReduceLargeIndicesTests, LargeIndices)
             unsigned int seed_value
                 = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
             SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
-            SCOPED_TRACE(testing::Message() << "with size = " << size);
+            SCOPED_TRACE(testing::Message() << "with size= " << size);
 
             hipStream_t stream = 0; // default
 
@@ -609,12 +912,12 @@ TYPED_TEST(HipcubDeviceReduceLargeIndicesTests, LargeIndices)
             void*  d_temp_storage = nullptr;
 
             // Get size of d_temp_storage
-            hipcub::DeviceReduce::Sum(d_temp_storage,
-                                      temp_storage_size_bytes,
-                                      d_input,
-                                      d_output,
-                                      size,
-                                      stream);
+            HIP_CHECK(hipcub::DeviceReduce::Sum(d_temp_storage,
+                                                temp_storage_size_bytes,
+                                                d_input,
+                                                d_output,
+                                                size,
+                                                stream));
 
             // temp_storage_size_bytes must be >0
             ASSERT_GT(temp_storage_size_bytes, 0U);
@@ -624,12 +927,12 @@ TYPED_TEST(HipcubDeviceReduceLargeIndicesTests, LargeIndices)
             HIP_CHECK(hipDeviceSynchronize());
 
             // Run
-            hipcub::DeviceReduce::Sum(d_temp_storage,
-                                      temp_storage_size_bytes,
-                                      d_input,
-                                      d_output,
-                                      size,
-                                      stream);
+            HIP_CHECK(hipcub::DeviceReduce::Sum(d_temp_storage,
+                                                temp_storage_size_bytes,
+                                                d_input,
+                                                d_output,
+                                                size,
+                                                stream));
             HIP_CHECK(hipPeekAtLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
