@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,7 @@
 #include "common_test_header.hpp"
 
 #include "hipcub/warp/warp_reduce.hpp"
+#include <type_traits>
 
 template<
     class T,
@@ -41,7 +42,7 @@ public:
     static constexpr unsigned int warp_size = Params::warp_size;
 };
 
-typedef ::testing::Types<
+using HipcubWarpReduceTestParams = ::testing::Types<
     // shuffle based reduce
     // Integer
     params<int, 1U>,
@@ -102,19 +103,15 @@ typedef ::testing::Types<
     params<float, 37U>,
     params<float, 61U>
 #endif
-    >
-    HipcubWarpReduceTestParams;
+    >;
 
 TYPED_TEST_SUITE(HipcubWarpReduceTests, HipcubWarpReduceTestParams);
 
-template<
-    class T,
-    unsigned int BlockSize,
-    unsigned int LogicalWarpSize
->
+template<class T, unsigned int BlockSize, unsigned int LogicalWarpSize>
 __global__
 __launch_bounds__(BlockSize)
-void warp_reduce_kernel(T* device_input, T* device_output)
+auto warp_reduce_kernel(T* device_input, T* device_output) ->
+    typename std::enable_if_t<test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
 {
     // Minimum size is 1
     constexpr unsigned int warps_no = test_utils::max(BlockSize / LogicalWarpSize, 1u);
@@ -134,6 +131,16 @@ void warp_reduce_kernel(T* device_input, T* device_output)
     }
 }
 
+template<class T, unsigned int BlockSize, unsigned int LogicalWarpSize>
+__global__
+__launch_bounds__(BlockSize)
+auto warp_reduce_kernel(T* /*device_input*/, T* /*device_output*/) ->
+    typename std::enable_if_t<!test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
+{
+    // This kernel should never be actually called; tests are filtered out at runtime
+    // if the device does not support the LogicalWarpSize
+}
+
 TYPED_TEST(HipcubWarpReduceTests, Reduce)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
@@ -257,14 +264,11 @@ TYPED_TEST(HipcubWarpReduceTests, Reduce)
     }
 }
 
-template<
-    class T,
-    unsigned int BlockSize,
-    unsigned int LogicalWarpSize
->
+template<class T, unsigned int BlockSize, unsigned int LogicalWarpSize>
 __global__
 __launch_bounds__(BlockSize)
-void warp_reduce_valid_kernel(T* device_input, T* device_output, const int valid)
+auto warp_reduce_valid_kernel(T* device_input, T* device_output, const int valid) ->
+    typename std::enable_if_t<test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
 {
     // Minimum size is 1
     constexpr unsigned int warps_no = test_utils::max(BlockSize / LogicalWarpSize, 1u);
@@ -284,6 +288,16 @@ void warp_reduce_valid_kernel(T* device_input, T* device_output, const int valid
     }
 }
 
+template<class T, unsigned int BlockSize, unsigned int LogicalWarpSize>
+__global__
+__launch_bounds__(BlockSize)
+auto warp_reduce_valid_kernel(T* /*device_input*/, T* /*device_output*/, const int /*valid*/) ->
+    typename std::enable_if_t<!test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
+{
+    // This kernel should never be actually called; tests are filtered out at runtime
+    // if the device does not support the LogicalWarpSize
+}
+
 TYPED_TEST(HipcubWarpReduceTests, ReduceValid)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
@@ -407,15 +421,11 @@ TYPED_TEST(HipcubWarpReduceTests, ReduceValid)
     }
 }
 
-template<
-    class T,
-    class Flag,
-    unsigned int BlockSize,
-    unsigned int LogicalWarpSize
->
+template<class T, class Flag, unsigned int BlockSize, unsigned int LogicalWarpSize>
 __global__
 __launch_bounds__(BlockSize)
-void head_segmented_warp_reduce_kernel(T* input, Flag* flags, T* output)
+auto head_segmented_warp_reduce_kernel(T* input, Flag* flags, T* output) ->
+    typename std::enable_if_t<test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
 {
     // Minimum size is 1
     constexpr unsigned int warps_no = test_utils::max(BlockSize / LogicalWarpSize, 1u);
@@ -432,6 +442,16 @@ void head_segmented_warp_reduce_kernel(T* input, Flag* flags, T* output)
     output[index] = value;
 }
 
+template<class T, class Flag, unsigned int BlockSize, unsigned int LogicalWarpSize>
+__global__
+__launch_bounds__(BlockSize)
+auto head_segmented_warp_reduce_kernel(T* /*input*/, Flag* /*flags*/, T* /*output*/) ->
+    typename std::enable_if_t<!test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
+{
+    // This kernel should never be actually called; tests are filtered out at runtime
+    // if the device does not support the LogicalWarpSize
+}
+
 TYPED_TEST(HipcubWarpReduceTests, HeadSegmentedReduceSum)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
@@ -610,15 +630,11 @@ TYPED_TEST(HipcubWarpReduceTests, HeadSegmentedReduceSum)
     }
 }
 
-template<
-    class T,
-    class Flag,
-    unsigned int BlockSize,
-    unsigned int LogicalWarpSize
->
+template<class T, class Flag, unsigned int BlockSize, unsigned int LogicalWarpSize>
 __global__
 __launch_bounds__(BlockSize)
-void tail_segmented_warp_reduce_kernel(T* input, Flag* flags, T* output)
+auto tail_segmented_warp_reduce_kernel(T* input, Flag* flags, T* output) ->
+    typename std::enable_if_t<test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
 {
     // Minimum size is 1
     constexpr unsigned int warps_no = test_utils::max(BlockSize / LogicalWarpSize, 1u);
@@ -636,6 +652,16 @@ void tail_segmented_warp_reduce_kernel(T* input, Flag* flags, T* output)
     output[index] = value;
 }
 
+template<class T, class Flag, unsigned int BlockSize, unsigned int LogicalWarpSize>
+__global__
+__launch_bounds__(BlockSize)
+auto tail_segmented_warp_reduce_kernel(T* /*input*/, Flag* /*flags*/, T* /*output*/) ->
+    typename std::enable_if_t<!test_utils::device_test_enabled_for_warp_size_v<LogicalWarpSize>>
+{
+    // This kernel should never be actually called; tests are filtered out at runtime
+    // if the device does not support the LogicalWarpSize
+}
+
 TYPED_TEST(HipcubWarpReduceTests, TailSegmentedReduceSum)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
