@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -39,6 +39,7 @@
 #include "test_utils_data_generation.hpp"
 #include "test_utils_functional.hpp"
 #include "test_utils_half.hpp"
+#include "test_utils_hipgraphs.hpp"
 #include "test_utils_sort_comparator.hpp"
 
 // Seed values
@@ -99,31 +100,31 @@ struct precision<custom_test_type<T>>
 template<class T>
 struct is_add_operator : std::false_type
 {
-    typedef uint8_t value_type;
+    using value_type = uint8_t;
 };
 
 template<class T>
 struct is_add_operator<test_utils::plus(T)> : std::true_type
 {
-    typedef T value_type;
+    using value_type = T;
 };
 
 template<class T>
 struct is_add_operator<test_utils::minus(T)> : std::true_type
 {
-    typedef T value_type;
+    using value_type = T;
 };
 
 template<class T>
 struct is_multiply_operator : std::false_type
 {
-    typedef uint8_t value_type;
+    using value_type = uint8_t;
 };
 
 template<class T>
 struct is_multiply_operator<test_utils::multiplies(T)> : std::true_type
 {
-    typedef T value_type;
+    using value_type = T;
 };
 
 /* Plus to operator selector for host-side
@@ -135,20 +136,26 @@ struct is_multiply_operator<test_utils::multiplies(T)> : std::true_type
 template<typename T>
 struct select_plus_operator_host
 {
-    typedef test_utils::plus type;
-    typedef T                acc_type;
+    using type     = test_utils::plus;
+    using acc_type = T;
 };
 
-template<class InputIt, class OutputIt, class BinaryOperation, class acc_type>
+template<bool UseInitialValue = false,
+         class InputIt,
+         class OutputIt,
+         class BinaryOperation,
+         class acc_type>
 OutputIt host_inclusive_scan_impl(
-    InputIt first, InputIt last, OutputIt d_first, BinaryOperation op, acc_type)
+    InputIt first, InputIt last, OutputIt d_first, BinaryOperation op, acc_type init_value)
 {
-    if(first == last)
-        return d_first;
-
-    acc_type sum = *first;
+    acc_type sum = UseInitialValue ? op(init_value, *first) : *first;
     *d_first     = sum;
 
+    if(first == last)
+    {
+        return d_first;
+    }
+
     while(++first != last)
     {
         sum        = op(sum, *first);
@@ -164,6 +171,13 @@ OutputIt host_inclusive_scan(InputIt first, InputIt last, OutputIt d_first, Bina
     return host_inclusive_scan_impl(first, last, d_first, op, acc_type{});
 }
 
+template<class InputIt, class OutputIt, class BinaryOperation, class InitType>
+OutputIt host_inclusive_scan_init(
+    InputIt first, InputIt last, OutputIt d_first, InitType init_value, BinaryOperation op)
+{
+    return host_inclusive_scan_impl<true>(first, last, d_first, op, init_value);
+}
+
 template<class InputIt,
          class OutputIt,
          class T,
@@ -181,6 +195,30 @@ OutputIt host_inclusive_scan(InputIt first, InputIt last, OutputIt d_first, test
     return host_inclusive_scan_impl(first, last, d_first, test_utils::plus(), acc_type{});
 }
 
+template<class InputIt,
+         class OutputIt,
+         class InitType,
+         class T,
+         std::enable_if_t<
+             std::is_same<typename std::iterator_traits<InputIt>::value_type,
+                          test_utils::bfloat16>::value
+                 || std::is_same<typename std::iterator_traits<InputIt>::value_type,
+                                 test_utils::half>::value
+                 || std::is_same<typename std::iterator_traits<InputIt>::value_type, float>::value,
+             bool>
+         = true>
+OutputIt host_inclusive_scan_init(
+    InputIt first, InputIt last, OutputIt d_first, InitType init_value, test_utils::plus)
+{
+    using acc_type = double;
+    return host_inclusive_scan_impl<true>(first,
+                                          last,
+                                          d_first,
+                                          init_value,
+                                          test_utils::plus(),
+                                          acc_type{});
+}
+
 template<class InputIt, class T, class OutputIt, class BinaryOperation, class acc_type>
 OutputIt host_exclusive_scan_impl(
     InputIt first, InputIt last, T initial_value, OutputIt d_first, BinaryOperation op, acc_type)
@@ -584,6 +622,14 @@ template<unsigned int LogicalWarpSize>
 __device__ constexpr bool device_test_enabled_for_warp_size_v
     = HIPCUB_DEVICE_WARP_THREADS >= LogicalWarpSize;
 
+template<typename T,
+         typename U,
+         std::enable_if_t<std::is_integral<T>::value && std::is_unsigned<U>::value, int> = 0>
+inline constexpr auto ceiling_div(const T a, const U b)
+{
+    return a / b + (a % b > 0 ? 1 : 0);
+}
+
 } // namespace test_utils
 
 // Need for hipcub::DeviceReduce::Min/Max etc.
